#include "CIF_Shared_State_def.H"
#include "globus_common.h"
#include <curses.h>

int up_index(char ch);

globus_mutex_t                done_mutex;
globus_cond_t                 done_cond;
globus_bool_t                 done;

class CIF_Slider : public CIF_Shared_State_Callback_Interface
{
private:
  int                      value;
  CIF_Shared_State_Int *   sharedInt;
  CIF_Shared_State_Set *   sharedSet;
  WINDOW *                 sliderWin;
  WINDOW *                 mainWin;
  int                      maxVal;
  int                      minVal;


  void draw();

public:

  CIF_Slider(WINDOW * parWin, int val, int max, int min, int x, int y);


  void    setValue(int val);
  int     getValue() {return value;};

  void    update();
  
  virtual void  
  state_Changed(
      CIF_Shared_State *                          shared_state,
      CIF_Shared_State_Changed_Type               type,
      CIF_Shared_State_Set_GUID *                 guid);

  void   
  startSharing(
      CIF_Shared_State_Set *                      set, 
      char *                                      name);
};

CIF_Slider::CIF_Slider(WINDOW * parWin, int val, int max, int min, int x, int y)
{
    maxVal = max;
    minVal = min;
    sliderWin = subwin(parWin, maxVal - minVal+2, 1, y, x);
//    nodelay(sliderWin, TRUE);
    mainWin = parWin;
    value = val;
    draw();
    sharedSet = NULL;
}

void
CIF_Slider::startSharing(CIF_Shared_State_Set * set, char * name)
{
   int a[1];

   a[0] = value;
   sharedSet = set;
   sharedInt = new CIF_Shared_State_Int(name, a, 1, this);

   set->add_Shared_State(sharedInt);
}

void 
CIF_Slider::setValue(int val)
{
  int a[1];
  a[0] = val;
  if (val < minVal || val > maxVal) return;
  value = val;
  if(sharedInt != NULL) 
  {
      sharedInt->set_State_Changed(0, a, 0, 1, FINAL);
  }
  draw();
}

void
CIF_Slider::state_Changed(
    CIF_Shared_State *                        shared_state,
    CIF_Shared_State_Changed_Type             type,
    CIF_Shared_State_Set_GUID *               guid)
{
    int a[1];

    if(guid->equals(sharedSet->get_GUID())) return;
    if(sharedInt)
    {
        sharedInt->get_State(a, 0, 0, 1);
    }
    value = a[0];
    draw();
}

void
CIF_Slider::draw()
{
    int ctr;

    for(ctr = 1; ctr < maxVal - minVal+1; ctr++)
    {
        mvwaddch(sliderWin, ctr, 0, '|');
    }
    mvwaddch(sliderWin, maxVal - value, 0, 'X');

    wrefresh(sliderWin);
}


void
CIF_Slider::update()
{
    redrawwin(sliderWin);
    wrefresh(sliderWin);
}

class Control_Panel 
{
private:
    CIF_Shared_State_Set * sharedSet;
    CIF_Slider *           sliders[10];
    WINDOW *               controlWin;
    WINDOW *               mainWin;
    int                    finishedFlag;

    void draw();

public:
    Control_Panel(WINDOW * parentWin, int x, int y, int height, int width);
    int update();

    void startSharing(CIF_Shared_State_Set * set);
};



Control_Panel::Control_Panel(WINDOW * win, int x, int y, int height, int width)
{
    int ctr;

    mainWin = win;

    finishedFlag = 0;
    controlWin = subwin(win, height, width, y, x);
    cbreak();
    nodelay(controlWin, TRUE);
    draw();

    for(ctr = 0; ctr < 10; ctr++)
    {
        sliders[ctr] = new CIF_Slider(controlWin, 5, 10, 1, ctr*3+x+1, 2+y);
    }

     redrawwin(controlWin);
     wrefresh(controlWin);
}

void
Control_Panel::startSharing(CIF_Shared_State_Set * set)
{
    int ctr;
    sharedSet = set;
    char *name[10] ={"S0", "S1", "S2", "S3", "S4",
                     "S5", "S6", "S7", "S8", "S9"};

    for(ctr = 0; ctr < 10; ctr++)
    {
        sliders[ctr]->startSharing(sharedSet, name[ctr]);
        globus_poll();
    }
    waddstr(controlWin, set->get_Message_Space_URL());
}

int 
Control_Panel::update()
{
    int ctr;
    char ch;
    int temp;
    int index;

    ch = getch();

    if(ch == ERR)
    {
        return -1;
    }
    else if(ch == 'q') 
    {
        done = GLOBUS_TRUE;
      return 1;
    }
    else if(ch >= '0' && ch <= '9')
    {
        if(ch == '0') index = 9;
        else
        {
            index = ch - 49;
        }
        temp = sliders[index]->getValue();
        temp++;
        sliders[index]->setValue(temp);
    }
    else 
    {
        index = up_index(ch);
        if(ch != -1)
        {
            temp = sliders[index]->getValue();
            temp--;
            sliders[index]->setValue(temp);
        }
    }

    wrefresh(controlWin);
    for(ctr = 0; ctr < 10; ctr++)
    {
        sliders[ctr]->update();
    }

    return 0;
}

void
Control_Panel::draw()
{
}

int main(int argc, char * argv[])
{
    CIF_Shared_State_Set *  set;
    WINDOW *                mainWin;
    Control_Panel *         cp;
    int                     err;


    CIF_Comm::activate();
    globus_mutex_init(&done_mutex, GLOBUS_NULL);
    globus_cond_init(&done_cond, GLOBUS_NULL);
    done = GLOBUS_FALSE;

    if(argc < 2)
    {
        set = new CIF_Shared_State_Set(CIF_MESSENGER_TYPE_TCP);
    }
    else
    {
        set = new CIF_Shared_State_Set(CIF_MESSENGER_TYPE_TCP, argv[1]);
    }


    mainWin = initscr();
    timeout(0);
    err = nodelay(mainWin, TRUE);
    if(err == ERR)
    {
        printf("bad nodlay sert\n");
        exit(1);
    }
    noecho();
    cp = new Control_Panel(mainWin, 1, 1, 20, 50);
    cp->startSharing(set);

    while(!done)
    {
        set->poll();
        globus_poll();
        if(cp->update() == 0)
        {
            refresh();
        }
    }

    endwin();
    set->close();
//    delete set;
    CIF_Comm::deactivate();
}

int up_index(char ch)
{
  char * ups = "!@#$%^&*()";
  int ctr;

  for(ctr = 0; ctr < 10; ctr++)
  {
     if(ch == ups[ctr]) return ctr;
  }

  return -1;
}
