/******************************************************************************
CIF_Comm_Factory.C

Description:

  CIF Object Factory

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/cpp/libraries/comm/CIF_Comm_Factory.C,v $
  $Date: 1999/11/15 22:51:31 $
  $Revision: 1.7 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_Comm.H"
#include "CIF_I_Comm_Connection_TCP.H"
#include "CIF_I_Comm_Connection_UDP.H"
#include "CIF_I_Comm_Connection_ROM.H"
#include "CIF_I_Comm_Connection_UUM.H"
#include "CIF_I_Comm_Listener_TCP.H"
#include "CIF_I_Comm_Listener_UDP.H"
#include "globus_common.h"

#include <string.h>


/******************************************************************************
			Define methods for CIF_Comm_Factory
******************************************************************************/

CIF_Comm_Connection *
CIF_Comm_Factory::get_Connection(
    ARG_IN const char *	const			url)
{
    return l_get_Connection(url,
			    CIF_Comm_Connection::Properties(),
			    CIF_Toolkit_Timeout::INFINITY);
}


CIF_Comm_Connection *
CIF_Comm_Factory::get_Connection(
    ARG_IN const char *	const			url,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    return l_get_Connection(url,
			    CIF_Comm_Connection::Properties(),
			    timeout);
}


CIF_Comm_Connection *
CIF_Comm_Factory::get_Connection(
    ARG_IN const char *	const			url,
    ARG_IN const
    CIF_Comm_Connection::Properties &		properties)
{
    return l_get_Connection(url,
			    properties,
			    CIF_Toolkit_Timeout::INFINITY);
}


CIF_Comm_Connection *
CIF_Comm_Factory::get_Connection(
    ARG_IN const char *	const			url,
    ARG_IN const
    CIF_Comm_Connection::Properties &		properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    return l_get_Connection(url,
			    properties,
			    timeout);
}

CIF_Comm_Connection *
CIF_Comm_Factory::l_get_Connection(
    ARG_IN const char *				url,
    ARG_IN const
    CIF_Comm_Connection::Properties &		properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    CIF_Comm_Connection *			conn;
    globus_url_t				url_info;

    if (globus_url_parse((char *) url, &url_info) != GLOBUS_SUCCESS)
    {
	CIF_Comm::status = CIF_Comm_Status::URL_FORMAT_BAD;
	return CIF_NULL;
    }

    if (strcmp(url_info.scheme,
	       CIF_COMM_PROTOCOL_TCP_STR) == 0)
    {
	conn = (CIF_Comm_Connection *) new CIF_I_Comm_Connection_TCP();
    }
    else if (strcmp(url_info.scheme,
		    CIF_COMM_PROTOCOL_UDP_STR) == 0)
    {
	conn = (CIF_Comm_Connection *) new CIF_I_Comm_Connection_UDP();
    }
    else if (strcmp(url_info.scheme,
		    CIF_COMM_PROTOCOL_ROM_STR) == 0)
    {
	conn = (CIF_Comm_Connection *) new CIF_I_Comm_Connection_ROM();
    }
/*    else if (strcmp(url_info.scheme,
		    CIF_COMM_PROTOCOL_RSM_STR) == 0)
    {
	conn = (CIF_Comm_Connection *) new CIF_I_Comm_Connection_RSM();
    }
*/    else if (strcmp(url_info.scheme,
		    CIF_COMM_PROTOCOL_UUM_STR) == 0)
    {
	conn = (CIF_Comm_Connection *) new CIF_I_Comm_Connection_UUM();
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::URL_SCHEME_UNSUPPORTED;
	return CIF_NULL;
    }
    
    if (CIF_Comm::status != CIF_Comm_Status::SUCCESS
	|| !conn->open(url, properties, timeout))
    {
	CIF_Comm_Status			status;

	status = CIF_Comm::status;
	delete conn;
	CIF_Comm::status = status;
	conn = CIF_NULL;
    }

    return conn;
}
    

CIF_Comm_Listener *
CIF_Comm_Factory::get_Listener(
    ARG_IN const char * const			url)
{
    return l_get_Listener(url, CIF_Comm_Listener::Properties());
}

CIF_Comm_Listener *
CIF_Comm_Factory::get_Listener(
    ARG_IN const char *	const			url,
    ARG_IN const
    CIF_Comm_Listener::Properties &		properties)
{
    return l_get_Listener(url, properties);
}


CIF_Comm_Listener *
CIF_Comm_Factory::l_get_Listener(
    ARG_IN const char *				url,
    ARG_IN const
    CIF_Comm_Listener::Properties &		properties)
{
    CIF_Comm_Listener *				listener;
    int						port;

    if (strncmp(url, CIF_COMM_PROTOCOL_TCP_STR,
		strlen(CIF_COMM_PROTOCOL_TCP_STR)) == 0)
    {
	url += strlen(CIF_COMM_PROTOCOL_TCP_STR);
	listener = (CIF_Comm_Listener *) new CIF_I_Comm_Listener_TCP();
    }
    else if (strncmp(url, CIF_COMM_PROTOCOL_UDP_STR,
		     strlen(CIF_COMM_PROTOCOL_UDP_STR)) == 0)
    {
	url += strlen(CIF_COMM_PROTOCOL_UDP_STR);
	listener = (CIF_Comm_Listener *) new CIF_I_Comm_Listener_UDP();
    }
    else
    {
	CIF_Comm::status = CIF_Comm_Status::URL_SCHEME_UNSUPPORTED;
	return CIF_NULL;
    }

    if (url[0] != ':')
    {
	delete listener;
	CIF_Comm::status = CIF_Comm_Status::URL_FORMAT_BAD;
	return CIF_NULL;
    }

    port = 0;
    if (url[1] != 0)
    {
	char				tmp_char;
	
	if (sscanf(url, "://%d%c", &port, &tmp_char) != 1 || port < 0)
	{
	    delete listener;
	    CIF_Comm::status = CIF_Comm_Status::URL_FORMAT_BAD;
	    return CIF_NULL;
	}
    }
    
    if (CIF_Comm::status != CIF_Comm_Status::SUCCESS ||
	listener->open(port, properties) == false)
    {
	CIF_Comm_Status			status;

	status = CIF_Comm::status;
	delete listener;
	CIF_Comm::status = status;
	listener = CIF_NULL;
    }

    return listener;
}

