/******************************************************************************
CIF_Comm_Listener.H

Description:

  An abstract base class from which all other CIF_Comm listener objects are
  derived.  The class also includes the defintions of events and properties
  associated with a connection.
  
CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/cpp/libraries/comm/CIF_Comm_Listener.H,v $
  $Date: 1998/10/09 01:00:31 $
  $Revision: 1.7 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

#if !defined(CIF_INCLUDE_COMM_LISTENER)
#define CIF_INCLUDE_COMM_LISTENER 1

/******************************************************************************
			   Define CIF_Comm_Listener class
******************************************************************************/
class
CIF_Comm_Listener
: public CIF_Toolkit_Object
{
    friend class CIF_Comm_Factory;

    /*
     * CIF_Comm_Listener::Connection_Request_Event
     *
     * Event delivered to registered event listener when a new connection
     * request is received.
     */
    public:
    class Connection_Request_Event
    : public CIF_Toolkit_Event
    {
	public:
	static const CIF_Toolkit_UUID		CLASS_ID;
	
	public:
	Connection_Request_Event(
	    CIF_Comm_Listener *			listener);

	public:
	CIF_Comm_Listener *
	get_Listener() const;

	private:
	CIF_Comm_Listener *			l_listener;
    };

    /*
     * CIF_Comm_Listener::Properties
     *
     * Properties to be associated with a listener.  Properties can only be
     * specified when the listener is opened and will remain in effect until
     * the listener is closed..
     *
     * At present, the only property is a pointer to an event listener which is
     * expect to receive a Connection_Request_Event whenever a new connection
     * request is received.  See CIF_I_Comm_Properties.H for methods associated
     * with the base properties class.
     *
     */
    public:
    class Properties
    : public CIF_I_Comm_Properties
    {
    };
    
    /*
     * CIF_Comm_Listener::open()
     *
     * Start listening for connection requests.  If a port number is specified,
     * the listener will be established on that port; otherwise, a port will be
     * assigned by the operating system.
     */
    protected: virtual
    CIF_Bool
    open(
	ARG_IN const unsigned short		port,
	ARG_IN const Properties &		properties) = 0;

    /*
     * CIF_Comm_Listener::get_Connection()
     *
     * Returns the next established connection.  If no connection is
     * established, this function blocks until such time as one is established
     * or the specified time out elapses.
     */
    public: virtual
    CIF_Comm_Connection *
    get_Connection() = 0;

    public: virtual
    CIF_Comm_Connection *
    get_Connection(
	ARG_IN const CIF_Toolkit_Timeout &	timeout) = 0;
    
    public: virtual
    CIF_Comm_Connection *
    get_Connection(
	ARG_IN const
	CIF_Comm_Connection::Properties &	properties) = 0;
    
    public: virtual
    CIF_Comm_Connection *
    get_Connection(
	ARG_IN const
	CIF_Comm_Connection::Properties &	properties,
	ARG_IN const
	CIF_Toolkit_Timeout &			timeout) = 0;
    
    /*
     * CIF_Comm_Listener::enable_Event_Listener()
     */
    public: virtual
    CIF_Bool
    enable_Event_Listener() = 0;

    /*
     * CIF_Comm_Listener::disable_Event_Listener()
     */
    public: virtual
    CIF_Bool
    disable_Event_Listener() = 0;

    /*
     * CIF_Comm_Listener::get_URL()
     *
     * Returns the full URL of the listener
     */
    public: virtual
    const char *
    get_URL() = 0;

    /*
     * CIF_Comm_Listener::close()
     *
     * Stop listening for connection requests
     * 
     */
    public: virtual
    CIF_Bool
    close() = 0;
};

#endif
