/******************************************************************************
CIF_I_Comm_Connection.C

Description:

  

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/cpp/libraries/comm/CIF_I_Comm_Connection.C,v $
  $Date: 1999/11/15 22:51:32 $
  $Revision: 1.11 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_I_Comm_Connection.H"

#include <stdio.h>
#include <string.h>


/******************************************************************************
		       CIF_I_Comm_Connection API methods
******************************************************************************/

/*
 * CIF_I_Comm_Connection::CIF_I_Comm_Connection()
 */
CIF_I_Comm_Connection::CIF_I_Comm_Connection()
{
    l_flow = CONNECTION_UNESTABLISHED;
    l_event_listener = CIF_NULL;
    l_blocking_cnt = 0;
    l_close_pending = false;
    l_object_state = OBJECT_STATE_INACTIVE;

    CIF_Comm::status = CIF_Comm_Status::SUCCESS;
}
/* CIF_I_Comm_Connection::CIF_I_Comm_Connection() */


/*
 * CIF_I_Comm_Connection::open(char *)
 */
CIF_Bool
CIF_I_Comm_Connection::open(
    ARG_IN const char *				url,
    ARG_IN const
    CIF_Comm_Connection::Properties &		properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that this is an inactive but valid connection object
     */
    CIF_Comm_Status				status;

    status = l_verify_Object_State(OBJECT_STATE_INACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    /*
     * Translate application supplied properties into endpoint properties
     */
    CIF_I_Comm_Endpoint::Properties		ep_properties;

    l_event_listener = properties.get_Event_Listener();
    if (l_event_listener != CIF_NULL)
    {
	ep_properties.set_Event_Listener(this);
    }

    /*
     * Establish the connection
     */
    l_flow = open_Connection(url, ep_properties, timeout);
    if (l_flow == CONNECTION_UNESTABLISHED)
    {
	l_event_listener = CIF_NULL;
	return false;
    }

    CIF_Comm::status = CIF_Comm_Status::SUCCESS;
    l_object_state = OBJECT_STATE_ACTIVE;
    return true;
}
/* CIF_I_Comm_Connection::open(char *) */


/*
 * CIF_I_Comm_Connection::open(globus_nexus_buffer_t &)
 */
CIF_Bool
CIF_I_Comm_Connection::open(
    ARG_INOUT globus_nexus_buffer_t &		nx_buf,
    ARG_IN const
    CIF_Comm_Connection::Properties &		properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    CIF_Toolkit_Monitor				monitor(l_mutex);

    /*
     * Verify that this is an inactive but valid connection object
     */
    CIF_Comm_Status				status;

    status = l_verify_Object_State(OBJECT_STATE_INACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    /*
     * Translate application supplied properties into endpoint properties
     */
    CIF_I_Comm_Endpoint::Properties		ep_properties;

    l_event_listener = properties.get_Event_Listener();
    if (l_event_listener != CIF_NULL)
    {
	ep_properties.set_Event_Listener(this);
    }

    /*
     * Establish the connection
     */
    l_flow = open_Connection(nx_buf, ep_properties, timeout);
    if (l_flow == CONNECTION_UNESTABLISHED)
    {
	l_event_listener = CIF_NULL;
	return false;
    }

    CIF_Comm::status = CIF_Comm_Status::SUCCESS;
    l_object_state = OBJECT_STATE_ACTIVE;
    return true;
}
/* CIF_I_Comm_Connection::open(CIF_I_Comm_Endpoint &) */


/*
 * CIF_I_Comm_Connection::send()
 */
CIF_Bool
CIF_I_Comm_Connection::send(
    ARG_IN const CIF_Byte *			msg_buf,
    ARG_IN const CIF_Size			msg_len)
{
    return l_send(msg_buf, msg_len, 0);
}


/*
 * CIF_I_Comm_Connection::send()
 */
CIF_Bool
CIF_I_Comm_Connection::send(
    ARG_IN const CIF_Byte *			msg_buf,
    ARG_IN const CIF_Size			msg_len,
    ARG_IN const int				msg_tag)
{
    return l_send(msg_buf, msg_len, msg_tag);
}


/*
 * CIF_I_Comm_Connection::l_send()
 */
CIF_Bool
CIF_I_Comm_Connection::l_send(
    ARG_IN const CIF_Byte *			msg_buf,
    ARG_IN const CIF_Size			msg_len,
    ARG_IN const int				msg_tag)
{
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	/*
	 * Verify that this is a valid connection object
	 */
	CIF_Comm_Status				status;

	status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
	if (status != CIF_Comm_Status::SUCCESS)
	{
	    CIF_Comm::status = status;
	    return false;
	}

	if ((l_flow & CONNECTION_OUTGOING) == 0)
	{
	    CIF_Comm::status = CIF_Comm_Status::FAILURE;
	    return false;
	}

	l_blocking_cnt++;
    }

    /*
     * Pack message into a Nexus buffer
     */
    globus_nexus_buffer_t			nx_buf;
    CIF_Size					nx_buf_sz;
    int						msg_len_int = msg_len;

    nx_buf_sz = globus_nexus_sizeof_int(2);
    nx_buf_sz += globus_nexus_sizeof_byte(msg_len);
    globus_nexus_buffer_init(&nx_buf, nx_buf_sz, 0);
    globus_nexus_put_int(&nx_buf, &msg_len_int, 1);
    globus_nexus_put_int(&nx_buf, (int *) &msg_tag, 1);
    globus_nexus_put_byte(&nx_buf, (globus_byte_t *) msg_buf, msg_len);

    /*
     * Send the buffer 
     */
    CIF_Bool 					rc;
    
    if (l_sp.send(nx_buf))
    {
	rc = true;
    }
    else
    {
	globus_nexus_buffer_destroy(&nx_buf);
	rc = false;
    }

    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	l_blocking_cnt--;
	if (l_close_pending && l_blocking_cnt == 0)
	{
	    l_close_cond.signal();
	}
    }

    return rc;
}
/* CIF_I_Comm_Connection::l_send() */


/*
 * CIF_I_Comm_Connection::l_receive()
 */
CIF_Bool
CIF_I_Comm_Connection::receive(
    ARG_INOUT CIF_Comm_Message &		message)
{
    return l_receive(message, CIF_Toolkit_Timeout::INFINITY);
}


/*
 * CIF_I_Comm_Connection::receive()
 */
CIF_Bool
CIF_I_Comm_Connection::receive(
    ARG_INOUT CIF_Comm_Message &		message,	
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    return l_receive(message, timeout);
}


/*
 * CIF_I_Comm_Connection::l_receive()
 */
CIF_Bool
CIF_I_Comm_Connection::l_receive(
    ARG_INOUT CIF_Comm_Message &		message,	
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	/*
	 * Verify that this is a valid connection object
	 */
	CIF_Comm_Status				status;

	status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
	if (status != CIF_Comm_Status::SUCCESS)
	{
	    CIF_Comm::status = status;
	    return false;
	}

	if ((l_flow & CONNECTION_INCOMING) == 0)
	{
	    CIF_Comm::status = CIF_Comm_Status::FAILURE;
	    return false;
	}

	l_blocking_cnt++;
    }
    
    CIF_Comm_Message_Info			msg_info;
    int						msg_size;
    int						msg_tag;
    
    globus_nexus_buffer_t			nx_buf;

    CIF_Bool					rc;

    if (l_ep.receive(nx_buf, timeout))
    {
	globus_nexus_get_int(&nx_buf, &msg_size, 1);
	globus_nexus_get_int(&nx_buf, &msg_tag, 1);
	
	msg_info.size = (CIF_Size) msg_size;
	msg_info.tag = msg_tag;

    	message.activate(msg_info);
	globus_nexus_get_byte(&nx_buf, message.get_Data(), msg_info.size);

	globus_nexus_buffer_destroy(&nx_buf);
	
	rc = true;
    }
    else
    {
	rc = false;
    }

    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	l_blocking_cnt--;
	if (l_close_pending && l_blocking_cnt == 0)
	{
	    l_close_cond.signal();
	}
    }

    return rc;
}
/* CIF_I_Comm_Connection::l_receive() */


/*
 * CIF_I_Comm_Connection::poll()
 */
CIF_Bool
CIF_I_Comm_Connection::poll()
{
    CIF_Toolkit_Monitor			monitor(l_mutex);

    /*
     * Verify that this is a valid connection object
     */
    CIF_Comm_Status				status;

    status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    if ((l_flow & CONNECTION_INCOMING) == 0)
    {
	CIF_Comm::status = CIF_Comm_Status::FAILURE;
	return false;
    }

    return l_ep.poll();
}
/* CIF_I_Comm_Connection::poll() */


/*
 * CIF_I_Comm_Connection::enable_Event_Listener()
 */
CIF_Bool
CIF_I_Comm_Connection::enable_Event_Listener()
{
    CIF_Bool					rc;
    
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	/*
	 * Verify that this is a valid connection object
	 */
	CIF_Comm_Status				status;

	status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
	if (status != CIF_Comm_Status::SUCCESS)
	{
	    CIF_Comm::status = status;
	    return false;
	}

	if ((l_flow & CONNECTION_INCOMING) == 0)
	{
	    CIF_Comm::status = CIF_Comm_Status::FAILURE;
	    return false;
	}

	l_blocking_cnt++;
    }

    rc = l_ep.enable_Event_Listener();
    
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	l_blocking_cnt--;
	if (l_close_pending && l_blocking_cnt == 0)
	{
	    l_close_cond.signal();
	}
    }

    return rc;
}


/*
 * CIF_I_Comm_Connection::disable_Event_Listener()
 */
CIF_Bool
CIF_I_Comm_Connection::disable_Event_Listener()
{
    CIF_Bool					rc;
    
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	/*
	 * Verify that this is a valid connection object
	 */
	CIF_Comm_Status				status;

	status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
	if (status != CIF_Comm_Status::SUCCESS)
	{
	    CIF_Comm::status = status;
	    return false;
	}

	if ((l_flow & CONNECTION_INCOMING) == 0)
	{
	    CIF_Comm::status = CIF_Comm_Status::FAILURE;
	    return false;
	}

	l_blocking_cnt++;
    }

    rc = l_ep.disable_Event_Listener();
    
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	l_blocking_cnt--;
	if (l_close_pending && l_blocking_cnt == 0)
	{
	    l_close_cond.signal();
	}
    }

    return rc;
}


/*
 * CIF_I_Comm_Connection::l_close()
 */
CIF_Bool
CIF_I_Comm_Connection::l_close(
    CIF_Toolkit_Monitor &			monitor)
{
    /*
     * Verify that this is a valid connection object
     */
    CIF_Comm_Status				status;

    status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
    if (status != CIF_Comm_Status::SUCCESS)
    {
	CIF_Comm::status = status;
	return false;
    }

    /*
     * If this is the first thread to attempt a close, then close down the
     * connection so that blocking operations will return
     */
    if (l_close_pending == false)
    {
	l_close_pending = true;

	if ((l_flow & CONNECTION_INCOMING) == CONNECTION_INCOMING)
	{
	    l_ep.deactivate();
	}

	if ((l_flow & CONNECTION_OUTGOING) == CONNECTION_OUTGOING)
	{
	    l_sp.deactivate();
	}
    }

    /*
     * Wait for all outstanding blocking operations to complete
     */
    while (l_blocking_cnt > 0)
    {
	monitor.wait();

	/*
	 * Verify that another thread hasn't closed the connection while we
	 * were sleeping
	 */
	status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
	if (status != CIF_Comm_Status::SUCCESS)
	{
	    CIF_Comm::status = status;
	    return false;
	}
    }

    /*
     * Reset state to indicate that the connection is closed
     */
    l_close_pending = false;
    l_event_listener = CIF_NULL;
    l_flow = CONNECTION_UNESTABLISHED;

    l_object_state = OBJECT_STATE_INACTIVE;

    /*
     * Wake up any remaining closers so that they can error out
     */
    l_close_cond.broadcast();

    return true;
}
/* CIF_I_Comm_Connection::l_close() */


/*
 * CIF_I_Comm_Connection::close()
 */
CIF_Bool
CIF_I_Comm_Connection::close()
{
    CIF_Toolkit_Monitor
	monitor(l_mutex, l_close_cond);

    return l_close(monitor);
}
/* CIF_I_Comm_Connection::close() */


/*
 * CIF_I_Comm_Connection::~CIF_I_Comm_Connection()
 */
CIF_I_Comm_Connection::~CIF_I_Comm_Connection()
{
    CIF_Comm_Status				status;
    
    CIF_Toolkit_Monitor
	monitor(l_mutex, l_close_cond);

    status = CIF_Comm::status;
    {
	l_close(monitor);
    
	l_object_state = OBJECT_STATE_DELETED;
    }
    CIF_Comm::status = status;
}
/* CIF_I_Comm_Connection::~CIF_I_Comm_Connection() */


/*
 * CIF_I_Comm_Connection::l_verify_Object_State()
 */
CIF_Comm_Status
CIF_I_Comm_Connection::l_verify_Object_State(
    ARG_IN const Object_State &			desired_state)
{
    if (l_object_state == desired_state)
    {
	return CIF_Comm_Status::SUCCESS;
    }
    
    /*
     * We aren't in the state we explected to be, so return an appropriate
     * error to the user
     */
    switch (l_object_state)
    {
      case OBJECT_STATE_DELETED:
	globus_assert(l_object_state != OBJECT_STATE_DELETED);
	return CIF_Comm_Status::OBJECT_DELETED;
	    
      case OBJECT_STATE_ACTIVE:
	return CIF_Comm_Status::OBJECT_ACTIVE;
	
      case OBJECT_STATE_INACTIVE:
	return CIF_Comm_Status::OBJECT_INACTIVE;

      default:
	return CIF_Comm_Status::INTERNAL_ERROR;
    }
}
/* CIF_I_Comm_Connection::l_verify_Object_State() */


/*
 * CIF_I_Comm_Connection::handle_Event()
 */
void
CIF_I_Comm_Connection::handle_Event(
    ARG_INMOD CIF_Toolkit_Event &		ep_event)
{
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	/*
	 * Verify that this is an active connection
	 */
	CIF_Comm_Status				status;

	status = l_verify_Object_State(OBJECT_STATE_ACTIVE);
	globus_assert(status == CIF_Comm_Status::SUCCESS);
	if (status != CIF_Comm_Status::SUCCESS)
	{
	    return;
	}

	l_blocking_cnt++;
    }

    Message_Received_Event			mr_event(this);
    
    l_event_listener->handle_Event(mr_event);
	
    {
	CIF_Toolkit_Monitor			monitor(l_mutex);

	l_blocking_cnt--;
	if (l_close_pending && l_blocking_cnt == 0)
	{
	    l_close_cond.signal();
	}
    }
}
/* CIF_I_Comm_Connection::handle_Event() */



