/******************************************************************************
CIF_I_Comm_Connection_RSM.C

Description:

  Connection class for Nexus using RSM / reliable source-ordered multicast

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/cpp/libraries/comm/CIF_I_Comm_Connection_RSM.C,v $
  $Date: 1999/03/28 14:17:40 $
  $Revision: 1.1 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_I_Comm_Connection_RSM.H"

#include <string.h>


/*
 * CIF_I_Comm_Connection_RSM::open_Connection()
 */
CIF_I_Comm_Connection::Connection_Flow_Mask
CIF_I_Comm_Connection_RSM::open_Connection(
    ARG_IN const char *				url,
    ARG_INMOD
    CIF_I_Comm_Endpoint::Properties &		ep_properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
#   if defined(HAVE_XTP_PROTO)
    {
	globus_nexus_proto_info_xtp_t		proto_info;
	globus_url_t				url_info;

	if (globus_url_parse((char *) url, &url_info) != GLOBUS_SUCCESS)
	{
	    CIF_Comm::status = CIF_Comm_Status::URL_FORMAT_BAD;
	    return CONNECTION_UNESTABLISHED;
	}

	if (strcmp(url_info.scheme, CIF_COMM_PROTOCOL_RSM_STR) != 0)
	{
	    globus_url_destroy(&url_info);

	    CIF_Comm::status = CIF_Comm_Status::URL_SCHEME_UNSUPPORTED;
	    return CONNECTION_UNESTABLISHED;
	}

	globus_url_destroy(&url_info);

	if (sscanf(url_info.url_path, "%d", &proto_info.process_group) != 1)
	{
	    globus_url_destroy(&url_info);

	    CIF_Comm::status = CIF_Comm_Status::URL_FORMAT_BAD;
	    return CONNECTION_UNESTABLISHED;
	}

	/*
	 * Create endpoint and startpoint
	 */
	ep_properties.set_Protocol(GLOBUS_NEXUS_PROTO_TYPE_XTP,
				   &proto_info,
				   sizeof(proto_info));
    
	if (l_ep.activate(ep_properties) == false)
	{
	    globus_url_destroy(&url_info);

	    CIF_Comm::status = CIF_Comm_Status::FAILURE;
	    return CONNECTION_UNESTABLISHED;
	}

	globus_url_destroy(&url_info);
    
	if (l_sp.activate(l_ep) == false)
	{
	    l_ep.deactivate();
	
	    CIF_Comm::status = CIF_Comm_Status::FAILURE;
	    return CONNECTION_UNESTABLISHED;
	}

	CIF_Comm::status = CIF_Comm_Status::SUCCESS;
	return CONNECTION_BIDIRECTIONAL;
    }
#   else
    {
	CIF_Comm::status = CIF_Comm_Status::URL_SCHEME_UNSUPPORTED;
	return CONNECTION_UNESTABLISHED;
    }
#   endif
}
/* CIF_I_Comm_Connection::open_Connection() */


/*
 * CIF_I_Comm_Connection_RSM::open_Connection()
 */
CIF_I_Comm_Connection::Connection_Flow_Mask
CIF_I_Comm_Connection_RSM::open_Connection(
    ARG_INOUT globus_nexus_buffer_t &		nx_buf,
    ARG_INMOD
    CIF_I_Comm_Endpoint::Properties &		ep_properties,
    ARG_IN const CIF_Toolkit_Timeout &		timeout)
{
    CIF_Comm::status = CIF_Comm_Status::FAILURE;
    return CONNECTION_UNESTABLISHED;
}
/* CIF_I_Comm_Connection_RSM::open_Connection() */
