/******************************************************************************
CIF_I_Comm_Startpoint.H

Description:

  Classes to provide simple Nexus startpoint functionality

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/cpp/libraries/comm/CIF_I_Comm_Startpoint.H,v $
  $Date: 1998/09/28 20:16:36 $
  $Revision: 1.5 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

#if !defined(CIF_INCLUDE_I_COMM_STARTPOINT)
#define CIF_INCLUDE_I_COMM_STARTPOINT 1

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF_Comm.H"
#include "CIF_I_Comm_Endpoint.H"

#include "globus_common.h"
#include "globus_nexus.h"


/******************************************************************************
			 Define CIF_I_Comm_Startpoint class
******************************************************************************/
class
CIF_I_Comm_Startpoint
: public CIF_Toolkit_Object
{
    /**************************************************************************
				    API methods
    **************************************************************************/

    /*
     * CIF_I_Comm_Startpoint::CIF_I_Comm_Startpoint()
     */
    public:
    CIF_I_Comm_Startpoint();

    /*
     * CIF_I_Comm_Startpoint::activate()
     */
    public:
    CIF_Bool
    activate(
	ARG_IN const char *			host,
	ARG_IN unsigned short			port);
    
    public:
    CIF_Bool
    activate(
	ARG_IN CIF_I_Comm_Endpoint &		ep);
    
    public:
    CIF_Bool
    activate(
	ARG_IN globus_nexus_buffer_t &		nx_buf);
    
    /*
     * CIF_I_Comm_Startpoint::send()
     */
    public:
    CIF_Bool
    send(
	ARG_INOUT globus_nexus_buffer_t &	nx_buffer);

    /*
     * CIF_I_Comm_Startpoint::sizeof_Startpoint()
     */
    public:
    CIF_Size
    sizeof_Startpoint();
    
    /*
     * CIF_I_Comm_Startpoint::put_Startpoint()
     */
    public:
    CIF_Bool
    put_Startpoint(
	ARG_INOUT globus_nexus_buffer_t &	nx_buffer);

    /*
     * CIF_I_Comm_Startpoint::deactivate()
     */
    public:
    CIF_Bool
    deactivate();
    
    /*
     * CIF_I_Comm_Startpoint::~CIF_I_Comm_Startpoint()
     */
    public:
    ~CIF_I_Comm_Startpoint();


    /**************************************************************************
		       Internal methods and data structures
    **************************************************************************/

    /*
     * State of the startpoint object
     */
    enum Object_State
    {
	OBJECT_STATE_INACTIVE,
	OBJECT_STATE_ACTIVE,
	OBJECT_STATE_DELETED
    };

    /*
     * CIF_I_Comm_Startpoint::l_deactivate()
     */
    private:
    CIF_Bool
    l_deactivate(
	CIF_Toolkit_Monitor &			monitor);
    
    /*
     * CIF_I_Comm_Startpoint::verify_Object_State()
     */
    private:
    CIF_Comm_Status
    l_verify_Object_State(
	ARG_IN Object_State			desired_state);
	
    /*
     * Internal data members
     */
    private:

    /* mutex to protect listener data structures */
    CIF_Toolkit_Mutex				l_mutex;

    /* nexus startpoint */
    globus_nexus_startpoint_t			l_nx_sp;
    
    /* state of the endpoint */
    Object_State				l_object_state;
};


#endif /* CIF_INCLUDE_I_COMM_STARTPOINT */
