#include "CIF_Shared_State_def.H"

////////////////////////////////////////////////////////////
//  CIF_I_List_Node
////////////////////////////////////////////////////////////

/* 
 *  CIF_I_List_Node::CIF_I_List_Node()
 */
CIF_I_List_Node::CIF_I_List_Node(void * in_element)
{
    next = NULL;
    element = in_element;
}

/* 
 *  CIF_I_List_Node::CIF_I_List_Node()
 */
CIF_I_List_Node::CIF_I_List_Node(void * in_element, char * in_key)
{
    next = NULL;
    element = in_element;
    key_Len = strlen(in_key)+1;
    key = new CIF_Byte[key_Len];
    memcpy((void *)key, (void *)in_key, key_Len);
}

/* 
 *  CIF_I_List_Node::CIF_I_List_Node()
 */
CIF_I_List_Node::CIF_I_List_Node(void * in_element, 
				 CIF_Byte * in_key, 
				 int key_len)
{
  next = NULL;
  element = in_element;
  key_Len = key_len;
  key = new CIF_Byte[key_Len];
  memcpy((void*)key, (void*)in_key, key_Len);
}

////////////////////////////////////////////////////////////
//  CIF_I_List
////////////////////////////////////////////////////////////

/* 
 *  CIF_I_List::CIF_I_List()
 */
CIF_I_List::CIF_I_List() : CIF_Shared_State_Base()
{
    first_Element = NULL;
    element_Count = 0;
}

CIF_I_List::~CIF_I_List()
{
    printf("destroying CIF_I_LIst\n");
}

/*  
 *  CIF_I_List::addElement()
 */
void
CIF_I_List::addElement(
    void *                                obj)
{
    lock();
    if(first_Element == NULL)
    {
        first_Element = new CIF_I_List_Node(obj);
        last_Element = first_Element;
        element_Count++;
    }
    else
    {
        last_Element->next = new CIF_I_List_Node(obj);
        last_Element = last_Element->next;
        element_Count++;
    }
    unlock();
}

/* 
 *  CIF_I_List::addElement()
 */
void
CIF_I_List::addElement(
    void *                              obj, 
    char *                              key)
{
    lock();
    if(first_Element == NULL)
    {
        first_Element = new CIF_I_List_Node(obj, key);
        last_Element = first_Element;
        element_Count++;
    }
    else
    {
        last_Element->next = new CIF_I_List_Node(obj, key);
        last_Element = last_Element->next;
        element_Count++;
    }
    unlock();
}

/* 
 *  CIF_I_List::addElement()
 */
void
CIF_I_List::addElement(void * obj, CIF_Byte * key, int key_len)
{
    lock();
    if(first_Element == NULL)
    {
        first_Element = new CIF_I_List_Node(obj, key, key_len);
        last_Element = first_Element;
        element_Count++;
    }
    else
    {
        last_Element->next = new CIF_I_List_Node(obj, key, key_len);
        last_Element = last_Element->next;
        element_Count++;
    }
    unlock();
}

/* 
 *  CIF_I_List::removeElement()
 */
CIF_Bool
CIF_I_List::removeElement(void * obj)
{
    CIF_I_List_Node * i;
    CIF_I_List_Node * j;
    int ctr;


    lock();
    if(first_Element == NULL)
    {  
        unlock();
        return (CIF_Bool)NEXUS_FALSE;
    }
    else if(first_Element->element == obj)
    {
        if(first_Element == last_Element) last_Element = NULL;
        first_Element = first_Element->next;
        element_Count--;
        unlock();
        return (CIF_Bool)NEXUS_TRUE;
    }


    lock();
    ctr = 0;
    j = first_Element;
    for(i = first_Element->next; i != NULL; i = i->next)
    {
        if(i->element == obj)
        {
            if(i == last_Element) last_Element = j;
            j->next = i->next; 
            element_Count--;
            unlock();
            return (CIF_Bool)NEXUS_TRUE;
        }
        ctr++;
        j = j->next;
    }

    unlock();
    return (CIF_Bool)NEXUS_FALSE;
}

/* 
 *  CIF_I_List::removeElement()
 */
CIF_Bool
CIF_I_List::removeElement(char * in_key)
{
    CIF_I_List_Node * i;
    CIF_I_List_Node * j;
    int ctr;

    lock();
    if(first_Element == NULL)
    {  
        unlock();
        return (CIF_Bool)NEXUS_FALSE;
    }
    else if(!memcmp(first_Element->key, in_key, first_Element->key_Len))
    {
        if(first_Element == last_Element) last_Element = NULL;
        first_Element = first_Element->next;
        element_Count--;
        unlock();
        return (CIF_Bool)NEXUS_TRUE;
    }

    ctr = 0;
    j = first_Element;
    for(i = first_Element->next; i != NULL; i = i->next)
    {
        if(!memcmp(i->key, in_key, i->key_Len))
        {
            if(i == last_Element) last_Element = j;
            j->next = i->next; 
            unlock();
            return (CIF_Bool)NEXUS_TRUE;
        }
        ctr++;
        j = j->next;
    }

    unlock();
    return (CIF_Bool)NEXUS_FALSE;
}

/* 
 *  CIF_I_List::removeElement()
 */
CIF_Bool
CIF_I_List::removeElement(
    CIF_Byte *                           in_key, 
    int                                  key_len)
{
    CIF_I_List_Node * i;
    CIF_I_List_Node * j;
    int ctr;

    lock();
    if(first_Element == NULL)
    {  
        unlock();
        return (CIF_Bool)NEXUS_FALSE;
    }
    else if(!memcmp(first_Element->key, in_key, first_Element->key_Len))
    {
        if(first_Element == last_Element) 
        {
	    last_Element = NULL;
        }   
        first_Element = first_Element->next;
        unlock();
        return (CIF_Bool)NEXUS_TRUE;
    }

    ctr = 0;
    j = first_Element;
    for(i = first_Element->next; i != NULL; i = i->next)
    {
        if(!memcmp(i->key, in_key, i->key_Len))
        {
            if(i == last_Element) last_Element = j;
            j->next = i->next; 
            unlock();
            return (CIF_Bool)NEXUS_TRUE;
        }
        ctr++;
        j = j->next;
    }
    unlock();
    return (CIF_Bool)NEXUS_FALSE;
}
/* 
 *  CIF_I_List::removeElementAt()
 */
CIF_Bool
CIF_I_List::removeElementAt(int index)
{
    CIF_I_List_Node * i;
    CIF_I_List_Node * j;
    int ctr;

    lock();
    if(index == 0)
    {
       if(first_Element != NULL)
       {
           if(first_Element == last_Element) last_Element = NULL;
           first_Element = first_Element->next;
           element_Count--;
           unlock();
           return (CIF_Bool)NEXUS_TRUE;
       }
       else
       {  
           unlock();
           return (CIF_Bool)NEXUS_FALSE;
       }
    }

    ctr = 1;
    j = first_Element;
    for(i = first_Element->next; i != NULL; i = i->next)
    {
        if(ctr == index)
        {
            if(i == last_Element) last_Element = j;
            j->next = i->next; 
            element_Count--;
            unlock();
            return (CIF_Bool)NEXUS_TRUE;
        }
        ctr++;
        j = j->next;
    }
    unlock();
    return (CIF_Bool)NEXUS_FALSE;
}

/* 
 *  CIF_I_List::removeElementAt()
 */
void
CIF_I_List::removeAllElements()
{
    lock();
    first_Element = NULL;
    element_Count = 0;
    unlock();
}

/*
 *  CIF_I_List::elementAt()
 */
void *
CIF_I_List::elementAt(int index)
{
    CIF_I_List_Node * i;
    int ctr = 0;

    lock();
    for(i = first_Element; i != NULL; i = i->next)
    {
        if(ctr == index)
        {
            unlock();
            return i->element;
        }
        ctr++;
    }
    unlock();
    return NULL;
}

/* 
 *  CIF_I_List::element()
 */
void *     
CIF_I_List::element(char * element_key)
{
    CIF_I_List_Node * i;
    int ctr = 0;

    lock();
    for(i = first_Element; i != NULL; i = i->next)
    {
        if(!memcmp(element_key, i->key, i->key_Len))
        {
            unlock();
            return i->element;
        }
        ctr++;
    }
    unlock();
    return NULL;
}

/* 
 *  CIF_I_List::element()
 */
void *     
CIF_I_List::element(CIF_Byte * element_key, int key_len)
{
    CIF_I_List_Node * i;
    int ctr = 0;

    lock();
    for(i = first_Element; i != NULL; i = i->next)
    {
        if(!memcmp(element_key, i->key, i->key_Len))
        {
            unlock();
            return i->element;
        }
        ctr++;
    }
    unlock();
    return NULL;
}

///////////////////////////////////////////////////////
//  CIF_I_Hash_Table
///////////////////////////////////////////////////////

/* 
 *  CIF_I_Hash_Table::CIF_I_Hash_Table()
 */
CIF_I_Hash_Table::CIF_I_Hash_Table(int table_size) : CIF_Shared_State_Base()
{
  table_Size = table_size;
  table_Array = new CIF_I_List[table_Size];
}

/* 
 *  CIF_I_Hash_Table::~CIF_I_Hash_Table()
 */
CIF_I_Hash_Table::~CIF_I_Hash_Table()
{
  delete table_Array;
}

/* 
 *  CIF_I_Hash_Table::getElement()
 */
void * 
CIF_I_Hash_Table::getElement(char * key)
{
  int hash_index;

  hash_index = calc_hash_index(key);
  return table_Array[hash_index].element(key);
}

/* 
 *  CIF_I_Hash_Table::getElement()
 */
void **
CIF_I_Hash_Table::getEnumeration()
{
  void ** objArray;
  int ctr;
  int ctr2;
  int count;
  int index;
 
    lock();
    count = getElementCount();
    objArray = (void**)malloc(count * sizeof(void*));

    index = 0;
    for(ctr = 0; ctr < table_Size; ctr++)
    {
        for(ctr2 = 0; ctr2 < table_Array[ctr].getElementCount(); ctr2++)
        {
            objArray[index] = table_Array[ctr].elementAt(ctr2);
            index++;
        }
    }
    unlock();
    return objArray;
}

/* 
 *  CIF_I_Hash_Table::putElement()
 */
void   
CIF_I_Hash_Table::putElement(char * key, void * obj)
{
    int hash_index;

    hash_index = calc_hash_index(key);

    lock();
    table_Array[hash_index].addElement(obj, key);
    unlock();
}

/* 
 *  CIF_I_Hash_Table::removeElement()
 */
void   
CIF_I_Hash_Table::removeElement(char * key)
{
    int hash_index;

    hash_index = calc_hash_index(key);

    lock();
    table_Array[hash_index].removeElement(key);
    unlock();
}

/* 
 *  CIF_I_Hash_Table::calc_hash_index()
 */
int  
CIF_I_Hash_Table::calc_hash_index(char * key)
{
    char *s;
    int hval = 0;

    for (s = key; *s != '\0'; s++)
    {
        hval <<= 1;
        hval += *s;
    }
  
    return hval % table_Size;
}


/* 
 *  CIF_I_Hash_Table::removeElement()
 */
int   
CIF_I_Hash_Table::getElementCount()
{
    int ctr;
    int count=0;

    lock();
    for(ctr = 0; ctr < table_Size; ctr++)
    {
        count += table_Array[ctr].getElementCount();
    }

    return count;
}

/* 
 *  CIF_I_Hash_Table::removeElement()
 */
void *   
CIF_I_Hash_Table::getElementNumber(int index)
{
    int ctr=0;
    int done = 0;
    int current_index=0;
    int last = 0;
 
    lock(); 
    for(ctr = 0; ctr < table_Size && !done; ctr++)
    {
        current_index += table_Array[ctr].getElementCount();
        if(index < current_index)
        {
            unlock(); 
            return table_Array[ctr].elementAt(index - last);
        }
        last = current_index;
    }

    lock(); 
    return NULL;
}
