#include "CIF_Shared_State_def.H"


globus_bool_t
close_callback(
    globus_abstime_t *                abstime,
    void *                            user_arg);

CIF_L_Connection::CIF_L_Connection(
    CIF_Messenger_TCP *              messenger, 
    CIF_Comm_Listener *              listener)
{
    CIF_Comm_Listener::Properties     properties_listener;
    char *                            tcp_url;
    char *                            uum_url;
    CIF_Comm_Connection::Properties   properties_connection;
    CIF_Shared_State_Serial *         serial = new CIF_Shared_State_Serial();
    int                               len[1];
    CIF_Byte *                        b;

    connection_Count = 0;

    properties_connection.set_Event_Listener(this);
    properties_listener.set_Event_Listener(this);
    state = NO_STATE;

    internal_Conn = listener->get_Connection(properties_connection,
 					    0);
    tcp_Listener = CIF_Comm_Factory::get_Listener("x-cif-tcp:",
						 properties_listener);
    uum_Listener = CIF_Comm_Factory::get_Listener("x-cif-tcp:",
						 properties_listener);

    tcp_url = (char *)tcp_Listener->get_URL();
    uum_url = (char *)uum_Listener->get_URL();

    len[0] = strlen(tcp_url);
    serial->pack_Int(len, 0, 1);
    serial->pack_Char(tcp_url, 0, len[0]);

    len[0] = strlen(uum_url);
    serial->pack_Int(len, 0, 1);
    serial->pack_Char(uum_url, 0, len[0]);

    messenger->total_Connections++;
    len[0] = messenger->total_Connections;
 
    serial->pack_Int(len, 0, 1);
    b = serial->get_Buffer();
    internal_Conn->send(b,
		       serial->get_Size_Buffer(),
		       0);
   
    internal_Conn->enable_Event_Listener();
    tcp_Listener->enable_Event_Listener();
    uum_Listener->enable_Event_Listener();

    tcp_Messenger = messenger;
}

CIF_L_Connection::CIF_L_Connection(
    CIF_Messenger_TCP *                   messenger,
    char *                                url)
{
    CIF_Comm_Connection::Properties        connection_properties;
    CIF_Comm_Message                       msg;
    CIF_Comm_Message_Info                  info;
    char *                                 tcp_url;
    char *                                 uum_url;
    CIF_Shared_State_Serial *              serial;
    int                                    len[1];

    connection_Count = 0;
    state = NO_STATE;
    tcp_Messenger = messenger;

    connection_properties.set_Event_Listener(this);

    internal_Conn = CIF_Comm_Factory::get_Connection(url, 
						    connection_properties);
    internal_Conn->receive(msg);
    info = msg.get_Info();
    serial = new CIF_Shared_State_Serial(msg.get_Data(), info.size);

    serial->unpack_Int(len, 0, 1);
    tcp_url = new char[len[0]+1];
    serial->unpack_Char(tcp_url, 0, len[0]);
    tcp_url[len[0]] = '\0';

    serial->unpack_Int(len, 0, 1);
    uum_url = new char[len[0]+1];
    serial->unpack_Char(uum_url, 0, len[0]);
    uum_url[len[0]] = '\0';

    serial->unpack_Int(len, 0, 1);
    tcp_Messenger->total_Connections = len[0];


    DebugPrint("about to get tcp connection");
    tcp_Conn = CIF_Comm_Factory::get_Connection(tcp_url,
					       connection_properties);
    DebugPrint("about to get uum connection");
    uum_Conn = CIF_Comm_Factory::get_Connection(uum_url,
					       connection_properties);

    DebugPrint("connections received");
    if(!tcp_Conn || !internal_Conn || !uum_Conn)
    {
        exit(1);
    }
    state = OPEN_STATE;
    internal_Conn->enable_Event_Listener();
    tcp_Conn->enable_Event_Listener();
    uum_Conn->enable_Event_Listener();

    delete tcp_url;
    delete uum_url;
    delete serial;
}

void 
CIF_L_Connection::enable(CIF_Bool b)
{

printf("CIF_L_Connection::enable() : start %d\n", b);
   if(b)
   {
      tcp_Conn->enable_Event_Listener();
   }
   else
   {
      tcp_Conn->disable_Event_Listener();
   }
}


void 
CIF_L_Connection::handle_Event(ARG_INMOD CIF_Toolkit_Event & event)
{
   CIF_Comm_Connection::Properties   properties_connection;
   CIF_Comm_Connection *             conn;
   CIF_Comm_Listener *               list;

   if(event.is_Member(CIF_Comm_Listener::Connection_Request_Event::CLASS_ID))
   {
     CIF_Comm_Listener::Connection_Request_Event * arrival_event = 
       (CIF_Comm_Listener::Connection_Request_Event *) &event;

     list = arrival_event->get_Listener();
     if(state != OPEN_STATE)
     {
        properties_connection.set_Event_Listener(this);

	if(list == tcp_Listener)
	{
	  tcp_Conn = tcp_Listener->get_Connection(properties_connection,
						  0);
          tcp_Conn->enable_Event_Listener();
        connection_Count++;
        }
	else if(list == uum_Listener)
	{
	  uum_Conn = uum_Listener->get_Connection(properties_connection,
						  0);
          uum_Conn->enable_Event_Listener();
        connection_Count++;
        }
	else
	{
          DebugPrint("list unknown");
	}
        if(connection_Count > 1) 
	{
            state = OPEN_STATE;
        }
     }
     else
     {
         DebugPrint("ERROR: connection is initialized");
     }
   }
   else if(event.is_Member(CIF_Comm_Connection::Message_Received_Event::CLASS_ID))
   {
       CIF_Comm_Message            msg;
       CIF_Comm_Message_Info       info;
       CIF_Byte *                  buf;
       CIF_Shared_State_Serial *   serial;
       CIF_Comm_Connection::Message_Received_Event  * arrival_event;

       arrival_event = (CIF_Comm_Connection::Message_Received_Event *) &event;

       if(state == OPEN_STATE)
       {       
          conn = arrival_event->get_Connection();
	  conn->receive(msg);
	  info = msg.get_Info();
	  buf = msg.get_Data();

	  serial = new CIF_Shared_State_Serial(buf, info.size);

	  if(conn == tcp_Conn)
	  {
             tcp_Messenger->receive_TCP(serial);
	  }
	  else if(conn == uum_Conn)
	  {
             tcp_Messenger->receive_UUM(serial);
	  }
	  else if(conn == internal_Conn)
	  {
             tcp_Messenger->receive_Internal(serial, this);
	  }
       }
   }
}

void 
CIF_L_Connection::send(
    CIF_Shared_State_Serial *            serial, 
    long                                 seq_num,
    CIF_L_Connection_Wire                wire)
{
    CIF_Comm_Connection *                conn;
    int                                  size;
    CIF_Byte *                           buf;
    long                                 value[1];

    if(state != OPEN_STATE)
    {
        return;
    }
    if(wire == INTERNAL_WIRE)
    {
        conn = internal_Conn;
    }
    else if(wire == TCP_WIRE)
    {
        conn = tcp_Conn;
    }
    else if(wire == UUM_WIRE)
    {
        conn = uum_Conn;
    }
    else
    {
        return;
    }

  buf = serial->get_Buffer();
  size = serial->get_Size_Buffer();

  value[0] = seq_num;
   //DataConversion.put_long(value, 0, 1, buf, 1);

   conn->send(buf, size);
}


CIF_L_Connection::~CIF_L_Connection()
{
    if(state == OPEN_STATE)
    {
        close();
    }
}

int
CIF_L_Connection::get_State()
{
    return state;
}

void
CIF_L_Connection::set_State(
    int                           in_state)
{
    in_state = state;
}

void
CIF_L_Connection::close()
{
    tcp_Conn->close();
    uum_Conn->close();
    internal_Conn->close();
    state = CLOSED_STATE;
    globus_mutex_lock(&tcp_Messenger->exit_mutex);
    {
        tcp_Messenger->can_Exit = (CIF_Bool) NEXUS_TRUE;
        globus_cond_signal(&tcp_Messenger->exit_cond);
    }
    globus_mutex_unlock(&tcp_Messenger->exit_mutex);
}

/**************************************************************************
*
**************************************************************************/
L_Nak_Node::L_Nak_Node(
    CIF_Shared_State *               state,
    int                              nak,
    CIF_Messenger_TCP *              mess,
    CIF_Byte *                       key,
    int                              key_length)
{
    shared_State = state;
    naks_Needed = nak;
    naks_Received = 0;

    key_Length = key_length;
    messenger = mess;
    shared_Set = messenger->shared_State_Set;
    key_Bytes = key;
    finished = (CIF_Bool)NEXUS_FALSE;
}


CIF_Bool 
L_Nak_Node::equals(L_Nak_Node * nak_node)
{
  char * in_name;
  CIF_Shared_State_Set_GUID * in_guid;

  in_name = nak_node->get_Name();
  in_guid = nak_node->get_GUID();

  if(!strcmp(in_name, shared_State->get_Name()) && 
     in_guid->equals(shared_Set->get_GUID()))
  {
      return (CIF_Bool)NEXUS_TRUE;
  }

  return (CIF_Bool)NEXUS_FALSE;
}

CIF_Bool                    
L_Nak_Node::add_Nak()
{
   naks_Received++;
   if(naks_Received < naks_Needed) return (CIF_Bool)NEXUS_FALSE;

   messenger->remove_Nak(shared_State->get_Name(), key_Bytes, key_Length);

   finished = (CIF_Bool) NEXUS_TRUE;

   return (CIF_Bool) NEXUS_TRUE;
}



CIF_Bool                    
L_Nak_Node::is_Done()
{
  if(naks_Received < naks_Needed) return (CIF_Bool)NEXUS_FALSE;

  return (CIF_Bool) NEXUS_TRUE;
}

/*****************************************************************/


/*
 *
 */
CIF_Messenger_TCP::CIF_Messenger_TCP(
    CIF_Shared_State_Set *              shared_state_set)
{
    CIF_Comm_Listener::Properties       properties_listener;

    total_Connections = 0;
    can_Exit = (CIF_Bool)NEXUS_TRUE;
    globus_mutex_init(&exit_mutex, GLOBUS_NULL);
    globus_cond_init(&exit_cond, GLOBUS_NULL);

    properties_listener.set_Event_Listener(this);
    shared_State_Set = shared_state_set;
    cif_Conn_List = new CIF_I_List();
    nak_List = new CIF_I_List();

    connection_Listener = CIF_Comm_Factory::get_Listener("x-cif-tcp:",
						  properties_listener);
    connection_Listener->enable_Event_Listener();
    space_URL = (char *)connection_Listener->get_URL();
    master_Flag = (CIF_Bool)NEXUS_TRUE;
}

CIF_Messenger_TCP::CIF_Messenger_TCP(
    CIF_Shared_State_Set *                    shared_state_set, 
    char *                                    url)
{
    CIF_L_Message_Header                      head;
    CIF_Byte *                                guid_bytes;
    int                                       len[1];
    CIF_Shared_State_Serial *                 serial;

    total_Connections = 0;
    can_Exit = (CIF_Bool)NEXUS_FALSE;
    globus_mutex_init(&exit_mutex, GLOBUS_NULL);
    globus_cond_init(&exit_cond, GLOBUS_NULL);

    shared_State_Set = shared_state_set;
    space_URL = strdup(url);

    nak_List = new CIF_I_List();
    slave_Conn = new CIF_L_Connection(this, url);
    master_Flag = (CIF_Bool)NEXUS_FALSE;

    head.type = INTERNAL_JOIN_MSG;
    head.seq_num = send_Number;

    guid_bytes = shared_State_Set->get_GUID()->get_Bytes();
    len[0] = shared_State_Set->get_GUID()->get_Length();
  
    serial = create_Serial_Header(head);
    serial->pack_Int(len, 0, 1);
    serial->pack_Byte(guid_bytes, 0, len[0]);
    len[0] = total_Connections;
    serial->pack_Int(len, 0, 1);

    distribute_Message(serial, INTERNAL_WIRE, NULL);
}

void         
CIF_Messenger_TCP::enable_Connections(
    CIF_Bool                                     b)
{
    int                                          ctr;
    CIF_L_Connection *                           l_conn;


    this->lock();
    {
        if(b) 
	{
	    times_Locked--;
        }
        if(times_Locked < 0) 
	{
	    times_Locked = 0;
	}

        if(times_Locked == 0)
        {
            if(master_Flag)
            {
                for(ctr = 0; ctr < cif_Conn_List->getElementCount(); ctr++)
	        {
                    l_conn = (CIF_L_Connection *) cif_Conn_List->elementAt(ctr);
	            l_conn->enable(b);
	        }
            }
            else
            {
                slave_Conn->enable(b);
            }
        }

        if(!b) 
	{
	    times_Locked++;
        }
    }
    this->unlock();
}

CIF_Shared_State_Serial *      
CIF_Messenger_TCP::create_Serial_Header(
    CIF_L_Message_Header                              head)
{
    CIF_Shared_State_Serial * serial = new CIF_Shared_State_Serial();

    create_User_Serial_Header(serial, head);

    return serial;
}

void
CIF_Messenger_TCP::create_User_Serial_Header(
    CIF_Shared_State_Serial *                           serial,
    CIF_L_Message_Header                                head)
{
    CIF_Byte  type[1];
    long      seq[1];

    type[0] = head.type;
    seq[0] = head.seq_num;
    serial->pack_Long(seq, 0, 1);
    serial->pack_Byte(type, 0, 1);
}

CIF_L_Message_Header           
CIF_Messenger_TCP::rip_Serial_Header(
    CIF_Shared_State_Serial *                         serial)
{
    CIF_L_Message_Header                              head;
    long                                              seq[1];
    CIF_Byte                                          type[1];

    serial->unpack_Long(seq, 0, 1);
    serial->unpack_Byte(type, 0, 1);

    head.type = type[0];
    head.seq_num = seq[0];

    return head;
}

void                      
CIF_Messenger_TCP::add_Shared_State(
    CIF_Shared_State *                      state)
{
  CIF_Shared_State_Serial *                 serial;
  CIF_Byte *                                guid_bytes;
  int                                       len[1];
  char *                                    name;
  L_Nak_Node *                              nak_node;
  CIF_Byte *                                key_bytes;
  CIF_L_Message_Header                      head;
  int                                       key_len;
  int                                       guid_len[1];

  head.type = INTERNAL_STATE_QUERY_MSG;
  head.seq_num = send_Number;
  serial = create_Serial_Header(head);
			  
  guid_bytes = shared_State_Set->get_GUID()->get_Bytes();
  guid_len[0] = shared_State_Set->get_GUID()->get_Length();

  serial->pack_Int(guid_len, 0, 1);
  serial->pack_Byte(guid_bytes, 0, guid_len[0]);

  name = state->get_Name();
  len[0] = strlen(name);
  serial->pack_Int(len, 0, 1);
  serial->pack_Char(name, 0, len[0]);


  if(master_Flag)
  {
    if(cif_Conn_List->getElementCount() == 0)
    {
       CIF_Shared_State_Serial * s;
       CIF_L_Message_Header                  h;

       serial = shared_State_Set->new_Shared_State_Serial(state);
       state->pack_State(serial);

       
       s = new CIF_Shared_State_Serial(serial->get_Buffer(), 
				       serial->get_Size_Buffer());
       h = rip_Serial_Header(s);
       DebugPrint("shared_State_Set = " << (void *) shared_State_Set);
       shared_State_Set->receive(s, INITIALIZE, h.seq_num);

       return;
    }
  }

  enable_Connections((CIF_Bool)NEXUS_FALSE);
  key_len = strlen(name) + shared_State_Set->get_GUID()->get_Length();
  key_bytes = new CIF_Byte[key_len];
  memcpy((void*)key_bytes, name, strlen(name));
  memcpy((void*)&key_bytes[strlen(name)], guid_bytes, guid_len[0]);

  nak_node = new L_Nak_Node(state, total_Connections, this, key_bytes, key_len);

  nak_List->addElement(nak_node, key_bytes, key_len);

  distribute_Message(serial, INTERNAL_WIRE, NULL);

  shared_State_Set->poll();
}


void                      
CIF_Messenger_TCP::send(CIF_Shared_State_Serial * serial,
		        CIF_Shared_State_Changed_Type type)
{
  CIF_L_Connection_Wire wire;

  if(type == FINAL)
  {
     wire = TCP_WIRE;
  }
  else
  {
     wire = UUM_WIRE;
  }

  if(master_Flag)
  {
     CIF_Shared_State_Serial * s;

     send_Number++;
     s = new CIF_Shared_State_Serial(serial->get_Buffer(), 
				     serial->get_Size_Buffer());
     rip_Serial_Header(s);
     shared_State_Set->receive(s, type, send_Number);
  }

  distribute_Message(serial, wire, NULL);
}

CIF_Shared_State_Serial * 
CIF_Messenger_TCP::new_Shared_State_Serial()
{
  CIF_Shared_State_Serial * serial;
  CIF_L_Message_Header head;

  head.type = STATE_CHANGE_MSG;
  head.seq_num = send_Number;
  serial = create_Serial_Header(head);

  return serial;
}

void 
CIF_Messenger_TCP::receive_TCP(CIF_Shared_State_Serial * serial)
{
  CIF_L_Message_Header head;

  if(master_Flag)
  {
     distribute_Message(serial, TCP_WIRE, NULL);
  }

  head = rip_Serial_Header(serial);

  last_Number = head.seq_num;
  shared_State_Set->receive(serial, FINAL, head.seq_num);
}

void 
CIF_Messenger_TCP::receive_UUM(CIF_Shared_State_Serial * serial)
{
   CIF_L_Message_Header head;

   head = rip_Serial_Header(serial);

   shared_State_Set->receive(serial, FINAL, head.seq_num);
}


void 
CIF_Messenger_TCP::receive_Internal(CIF_Shared_State_Serial * serial, 
	                            CIF_L_Connection * l_conn)
{
   CIF_Byte *                     guid_bytes;
   CIF_Shared_State_Set_GUID *    guid;
   CIF_Shared_State *             state;
   int                            guid_len[1];
   int                            name_len[1];
   char *                         name;
   CIF_Shared_State_Serial *      s;
   int                            ack_name_len[1];
   char *                         ack_name;
   int                            nak_name_len[1];
   char *                         nak_name;
   int                            c_count[1];
   CIF_Byte *                     key_bytes;
   CIF_L_Message_Header           nak_header;
   CIF_L_Message_Header           ack_header;
   CIF_L_Message_Header           kill_header;
   CIF_L_Message_Header           header;
   L_Nak_Node *                   nak_node;
   int                            key_len;
   globus_reltime_t               delay_time;

  if(master_Flag)
  {
     distribute_Message(serial, INTERNAL_WIRE, l_conn);
  }

  header = rip_Serial_Header(serial);
  serial->unpack_Int(guid_len, 0, 1);
  guid_bytes = new CIF_Byte[guid_len[0]];
  serial->unpack_Byte(guid_bytes, 0, guid_len[0]);
  guid = new CIF_Shared_State_Set_GUID(guid_bytes, guid_len[0]);


  switch(header.type)
  {
    case INTERNAL_STATE_QUERY_MSG:

       serial->unpack_Int(name_len, 0, 1);
       name = new char[name_len[0]+1];
       serial->unpack_Char(name, 0, name_len[0]);
       name[name_len[0]] = '\0';

       state = shared_State_Set->get_Shared_State(name);
       if(state != NULL)
       {
	  ack_header.type = INTERNAL_STATE_ACK_MSG;
	  ack_header.seq_num = send_Number;

	  s = create_Serial_Header(ack_header);
          s->pack_Int(guid_len, 0, 1);
	  s->pack_Byte(guid_bytes, 0, guid_len[0]);
	  s->pack_Int(name_len, 0, 1);
	  s->pack_Char(name, 0, name_len[0]);
	  shared_State_Set->pack_User_Set(s, state);
	  state->pack_State(s);

	  distribute_Message(s, INTERNAL_WIRE, NULL);
      
	  delete s;
       }
       else // send out a nak
       {
	  nak_header.type = INTERNAL_STATE_NAK_MSG;
	  nak_header.seq_num = send_Number;

          s = create_Serial_Header(nak_header);
	  s->pack_Int(guid_len, 0, 1);
	  s->pack_Byte(guid_bytes, 0, guid_len[0]);
	  s->pack_Int(name_len, 0, 1);
	  s->pack_Char(name, 0, name_len[0]);

	  distribute_Message(s, INTERNAL_WIRE, NULL);
	  delete s;
       }
     
       break;

    case INTERNAL_STATE_ACK_MSG:

      serial->unpack_Int(ack_name_len, 0, 1);
      ack_name = new char[ack_name_len[0]+1];
      serial->unpack_Char(ack_name, 0, ack_name_len[0]);
      ack_name[ack_name_len[0]+1] = '\0';

      key_len = ack_name_len[0]+guid_len[0];
      key_bytes = new CIF_Byte[key_len];
      memcpy(key_bytes, ack_name, ack_name_len[0]);
      memcpy(&key_bytes[ack_name_len[0]], guid_bytes, guid_len[0]);

      nak_node = (L_Nak_Node *)nak_List->element(key_bytes, key_len);
      if(nak_node != NULL)
      {
         nak_List->removeElement((void *)nak_node);
	 shared_State_Set->receive(serial,
				   INITIALIZE,
				   header.seq_num);

	 enable_Connections((CIF_Bool)NEXUS_TRUE);
      }
      
      break;

   case INTERNAL_STATE_NAK_MSG:

      serial->unpack_Int(nak_name_len, 0, 1);
      nak_name = new char[nak_name_len[0]+1];
      serial->unpack_Char(nak_name, 0, nak_name_len[0]);
      nak_name[nak_name_len[0]+1] = '\0';

      key_len = nak_name_len[0]+guid_len[0];
      key_bytes = new CIF_Byte[key_len];
      memcpy(key_bytes, nak_name, nak_name_len[0]);
      memcpy(&key_bytes[nak_name_len[0]], guid_bytes, guid_len[0]);

      nak_node = (L_Nak_Node *)nak_List->element(key_bytes, key_len);
      if(nak_node != NULL)
      {
        nak_node->add_Nak();
      }
     break;

   case INTERNAL_JOIN_MSG:

     serial->unpack_Int(c_count, 0, 1);
     if(!master_Flag)
     {
        total_Connections = c_count[0];
     }
     else
     {
//        total_Connections++;
     }
     shared_State_Set->set_Joined(guid);

     break;

    case INTERNAL_EXIT_MSG:

      shared_State_Set->set_Exited(guid);
      if(master_Flag)
      {
          kill_header.type = INTERNAL_CONFIRM_EXIT_MSG;
          kill_header.seq_num = send_Number;
          s = create_Serial_Header(kill_header);

          guid_len[0] = shared_State_Set->get_GUID()->get_Length();
          guid_bytes = shared_State_Set->get_GUID()->get_Bytes();
          s->pack_Int(guid_len, 0, 1);
          s->pack_Byte(guid_bytes, 0, guid_len[0]);

          total_Connections--;
          cif_Conn_List->removeElement(l_conn);
          l_conn->send(s, 3200, INTERNAL_WIRE);

          delete l_conn;
          delete s;
      }
      break;

    case INTERNAL_CONFIRM_EXIT_MSG:

      GlobusTimeReltimeSet(delay_time, 0, 0);
      l_conn->set_State(CLOSING_STATE);

printf("got confirm exit message\n");
//          l_conn->close();
      globus_callback_register_oneshot(
          GLOBUS_NULL,
          &delay_time,
          close_callback,
          (void *) l_conn,
          GLOBUS_NULL,
          GLOBUS_NULL);

      break;
  }
}

void                           
CIF_Messenger_TCP::remove_Nak(char * name, 
		               CIF_Byte * key_bytes,
			       int key_length)
{
   CIF_Shared_State * state;
   CIF_Shared_State_Serial * serial;
   CIF_Shared_State_Serial * s;
   CIF_L_Message_Header header;

   state = shared_State_Set->get_Shared_State(name);
   serial = shared_State_Set->new_Shared_State_Serial(state);
   state->pack_State(serial);

   s = new CIF_Shared_State_Serial(serial->get_Buffer(),
				   serial->get_Size_Buffer());
   header = rip_Serial_Header(s);
   shared_State_Set->receive(s,
			     NAK_INITIALIZE,
			     header.seq_num);

   nak_List->removeElement(key_bytes, key_length);

   enable_Connections((CIF_Bool)NEXUS_TRUE);
}

void 
CIF_Messenger_TCP::distribute_Message(
    CIF_Shared_State_Serial *                       serial,
    CIF_L_Connection_Wire                           wire,
    CIF_L_Connection *                              skip_conn)
{
    int ctr;
    CIF_L_Connection * l_conn;

    if(!master_Flag)
    {
        slave_Conn->send(serial, send_Number, wire);
        return;
    }

    for(ctr = 0; ctr < cif_Conn_List->getElementCount(); ctr++)
    {
        l_conn = (CIF_L_Connection *)cif_Conn_List->elementAt(ctr);
        if(l_conn != skip_conn)
        {
            l_conn->send(serial, send_Number, wire);
        }
    }
}


void 
CIF_Messenger_TCP::handle_Event(
    ARG_INMOD CIF_Toolkit_Event &                  event)
{
    CIF_L_Connection *                             l_conn;

    if(event.is_Member(CIF_Comm_Listener::Connection_Request_Event::CLASS_ID))
    {
        l_conn = new CIF_L_Connection(this, connection_Listener);
        cif_Conn_List->addElement(l_conn);
    }
    else
    {
        DebugPrint("unknow cif comm event occured");
    }
}

void 
CIF_Messenger_TCP::close()
{
    CIF_Byte * guid_bytes;
    int len[1];
    CIF_Shared_State_Serial * serial;
    CIF_L_Message_Header head;

printf("CIF_Messenger_TCP::close() : start\n");
    guid_bytes = shared_State_Set->get_GUID()->get_Bytes();
    len[0] = shared_State_Set->get_GUID()->get_Length();
    head.type = INTERNAL_EXIT_MSG;
    head.seq_num = send_Number;
    serial = create_Serial_Header(head);
    serial->pack_Int(len, 0, 1);
    serial->pack_Byte(guid_bytes, 0, len[0]);

    if(!master_Flag)
    {
        slave_Conn->send(serial, 3, INTERNAL_WIRE);
    }
printf("(locking)waiting ot exit\n");
    globus_mutex_lock(&exit_mutex);
    {
        while(!can_Exit)
        {
printf("waiting to exit\n");
            globus_cond_wait(&exit_cond, &exit_mutex);
        }
    }
    globus_mutex_unlock(&exit_mutex);

   if(!master_Flag) 
   {
//       slave_Conn->close();
   }
}

CIF_Messenger_TCP::~CIF_Messenger_TCP()
{
    globus_mutex_destroy(&exit_mutex);
    globus_cond_destroy(&exit_cond);
    close();
}


CIF_Bool                  
CIF_Messenger_TCP::poll()
{
   CIF_Comm::poll();

   return (CIF_Bool)NEXUS_TRUE;
}

globus_bool_t
close_callback(
    globus_abstime_t *                abstime,
    void *                            user_arg)
{
    CIF_L_Connection *                l_conn;

printf("close callback called\n");
    l_conn = (CIF_L_Connection *) user_arg;

    l_conn->close();
    delete l_conn;
printf("close callback ending\n");
}
