#ifndef __CIF_MESSENGER_TCP
#define __CIF_MESSENGER_TCP

#include "CIF_Shared_State_def.H"

class CIF_Messenger_TCP;

enum CIF_L_Connection_Wire
{
   INTERNAL_WIRE,
   TCP_WIRE,
   UUM_WIRE
};


enum CIF_L_Connection_State
{
    CLOSING_STATE,
    CLOSED_STATE,
    OPEN_STATE,
    NO_STATE
};

enum CIF_L_TCP_Message_Type
{
   STATE_CHANGE_MSG,

   INTERNAL_STATE_QUERY_MSG,
   INTERNAL_JOIN_MSG,
   INTERNAL_EXIT_MSG,
   INTERNAL_CONFIRM_EXIT_MSG,

   MESSAGE_DROPPED_MSG,

   INTERNAL_STATE_ACK_MSG,
   INTERNAL_STATE_NAK_MSG,
   INTERNAL_STATE_DUMP_REQUEST,
   INTERNAL_DUMP_ALL_REQUEST
};

typedef struct _CIF_L_Message_Header
{
  long                   seq_num;
  CIF_Byte               type;
} CIF_L_Message_Header;


class CIF_L_Connection : public CIF_Toolkit_Event_Listener
{
private:
   CIF_Comm_Connection *               internal_Conn;
   CIF_Comm_Connection *               tcp_Conn;
   CIF_Comm_Connection *               uum_Conn;
   int                                 state;
   int                                 connection_Count; 
   CIF_Comm_Listener *                 tcp_Listener;
   CIF_Comm_Listener *                 uum_Listener;

public:
   CIF_Messenger_TCP *                 tcp_Messenger;

   CIF_L_Connection(CIF_Messenger_TCP * messenger, 
 		   CIF_Comm_Listener * listener);

   CIF_L_Connection(CIF_Messenger_TCP * messenger,
		   char * url);

   ~CIF_L_Connection();

   void enable(CIF_Bool b);

   void close();
   void send(CIF_Shared_State_Serial * serial, 
	    long seq_num,
	    CIF_L_Connection_Wire wire);

   void set_State(
        int                              state);
   int get_State();

   virtual void handle_Event(ARG_INMOD CIF_Toolkit_Event & event);
};

class L_Nak_Node
{
protected:
   int                       naks_Needed;
   int                       naks_Received;
   int                       key_Length;
   CIF_Shared_State *        shared_State;
   CIF_Shared_State_Set *    shared_Set;
   CIF_Messenger_TCP *       messenger;
   CIF_Byte *                key_Bytes;
   CIF_Bool                  finished;

public:
  L_Nak_Node(CIF_Shared_State * state,
	     int nak,
	     CIF_Messenger_TCP * mess,
	     CIF_Byte * key,
	     int key_length);

  CIF_Bool equals(L_Nak_Node * nak_node);
  CIF_Byte * get_ID() {return key_Bytes;};
  CIF_Shared_State_Set_GUID * get_GUID() {return shared_Set->get_GUID();};
  char *                      get_Name() {return shared_State->get_Name();};
  CIF_Bool                    add_Nak();
  CIF_Bool                    is_Done();
};

class CIF_Messenger_TCP : public CIF_Shared_State_Messenger_Interface,
			  public CIF_Toolkit_Event_Listener,
                          public CIF_Shared_State_Base
{
 friend class L_Nak_Node;
protected:
  CIF_Shared_State_Set *             shared_State_Set;
  char *                             space_URL;
  CIF_Comm_Listener *                connection_Listener;

  CIF_Bool                           master_Flag;
  CIF_I_List *                       cif_Conn_List;
  CIF_I_List *                       nak_List;
  int                                message_Number[1];

  CIF_Comm_Listener::Properties      properties_Listener;

  CIF_L_Connection *                 slave_Conn;


  int                                times_Locked;
  long                               send_Number;
  long                               last_Number;

public:
  CIF_Bool                           can_Exit;
  globus_mutex_t                     exit_mutex;
  globus_cond_t                      exit_cond;
  int                                total_Connections;

protected:
  void handle_New_Connection();
  void distribute_Message(CIF_Shared_State_Serial * serial,
			  CIF_L_Connection_Wire wire,
			  CIF_L_Connection * from_conn);

  CIF_Shared_State_Serial *      create_Serial_Header(CIF_L_Message_Header head);
  void                           create_User_Serial_Header(CIF_Shared_State_Serial * serial,
							   CIF_L_Message_Header head);
  CIF_L_Message_Header           rip_Serial_Header(CIF_Shared_State_Serial * serial);
  void                           remove_Nak(char * name, 
					    CIF_Byte * key_bytes,
					    int key_length);
public:
  CIF_Messenger_TCP(CIF_Shared_State_Set * shared_state_set);
  CIF_Messenger_TCP(CIF_Shared_State_Set * shared_state_set, 
		    char * url);
  virtual ~CIF_Messenger_TCP();

  void         enable_Connections(CIF_Bool b);

  virtual void handle_Event(ARG_INMOD CIF_Toolkit_Event & event);

  void receive(CIF_Shared_State_Serial * serial, 
               CIF_L_Connection_Wire wire,
	       CIF_L_Connection * l_conn);

  void receive_Internal(CIF_Shared_State_Serial * serial, 
	                CIF_L_Connection * l_conn);
  void receive_TCP(CIF_Shared_State_Serial * serial); 
  void receive_UUM(CIF_Shared_State_Serial * serial); 

  char * get_Message_Space_URL()
  {
    return space_URL;
  }

  CIF_Bool                  poll();
  void                      send(CIF_Shared_State_Serial * serial,
				 CIF_Shared_State_Changed_Type type);

  CIF_Shared_State_Serial * new_Shared_State_Serial();
  void                      add_Shared_State(CIF_Shared_State * state);

  void close();
};

#endif
