#include "CIF_Shared_State_def.H"

/*
 *  CIF_Shared_State::CIF_Shared_State()
 */
CIF_Shared_State::CIF_Shared_State(
    char *                                 name)
{
    shared_State_Set = NULL;
    initialized = (CIF_Bool)GLOBUS_FALSE; 
  
    shared_State_Name = strdup(name);

    callback_List = new CIF_I_List();
    globus_mutex_init(&mutex, GLOBUS_NULL);
    globus_cond_init(&cond, GLOBUS_NULL);
}

/*
 *  CIF_Shared_State::CIF_Shared_State()
 */
CIF_Shared_State::CIF_Shared_State(
    char *                                   name,	
    CIF_Shared_State_Callback_Interface *    callback)
{
    shared_State_Set = NULL;
    initialized = (CIF_Bool)GLOBUS_FALSE; 
  
    shared_State_Name = strdup(name);

    callback_List = new CIF_I_List();

    globus_mutex_init(&mutex, GLOBUS_NULL);
    globus_cond_init(&cond, GLOBUS_NULL);
    add_Callback(callback);
}

/*
 *  CIF_Shared_State::~CIF_Shared_State()
 */
CIF_Shared_State::~CIF_Shared_State()
{
    free(shared_State_Name);

//    delete callback_List;
}

char *
CIF_Shared_State::get_Name()
{
    return shared_State_Name;
}

CIF_Bool
CIF_Shared_State::get_Initialized()
{
    return initialized;
}

void
CIF_Shared_State::set_Initialized(
    CIF_Bool                          b)
{
    lock();

    initialized = b;
    if(b)
    {
        notify();
    }
    unlock();
}

/*
 *
 */
void
CIF_Shared_State::joined_Set(
    CIF_Shared_State_Set *                    set)
{
    shared_State_Set = set;
}

/*
 *  CIF_Shared_State::add_Callback()
 */
void   
CIF_Shared_State::add_Callback(
    CIF_Shared_State_Callback_Interface *    callback)
{
    lock();
    {
        callback_List->addElement((void *)callback);
    }
    unlock();
}

/*
 *  CIF_Shared_State::remove_Callback()
 */
void   
CIF_Shared_State::remove_Callback(
    CIF_Shared_State_Callback_Interface *    callback)
{
    lock();
    {
        callback_List->removeElement((void *)callback);
    }
    unlock();
}

/*
 *  CIF_Shared_State::state_Changed()
 */
void  
CIF_Shared_State::state_Changed(
    CIF_Shared_State_Serial *                serial,
    CIF_Shared_State_Changed_Type            type,
    CIF_Shared_State_Set_GUID *              guid)
{
    int size;
    int ctr;
    CIF_Shared_State_Callback_Interface  * callback;


    lock();
    {
        unpack_State(serial);

	if(type != NAK_INITIALIZE)
        {
            size = callback_List->getElementCount();

            for(ctr = 0; ctr < size; ctr++)
            {
                callback = (CIF_Shared_State_Callback_Interface *) 
                             callback_List->elementAt(ctr);
                callback->state_Changed(this, type, guid);
            }	   
        }
    }
    unlock();
}
