#include "CIF_Shared_State_def.H"

/*
 *  CIF_Shared_State_Bool::CIF_Shared_State_Bool()
 */
CIF_Shared_State_Bool::CIF_Shared_State_Bool(
    char *                                        name,	
    int                                           size)
 : CIF_Shared_State(name)
{
    state_Value = new CIF_Bool[size];
    state_Count = size;
}

/*
 *  CIF_Shared_State_Bool::CIF_Shared_State_Bool()
 */
CIF_Shared_State_Bool::CIF_Shared_State_Bool(
    char *                                          name,	
    int                                             size,
    CIF_Shared_State_Callback_Interface *         callback)
 : CIF_Shared_State(name, callback)
{
    state_Value = new CIF_Bool[size];
    state_Count = size;
}

/*
 *  CIF_Shared_State_Bool::CIF_Shared_State_Bool()
 */
CIF_Shared_State_Bool::CIF_Shared_State_Bool(
    char *                                       name,
    CIF_Bool                                        init_value[],
    int                                          size)
 : CIF_Shared_State(name)
{
    state_Value = new CIF_Bool[size];
    memcpy(state_Value, init_value, sizeof(CIF_Bool) * size);
    state_Count = size;
}

/*
 *  CIF_Shared_State_Bool::CIF_Shared_State_Bool()
 */
CIF_Shared_State_Bool::CIF_Shared_State_Bool(
    char *                                         name,
    CIF_Bool                                          init_value[],
    int                                            size,
    CIF_Shared_State_Callback_Interface *          callback)
 : CIF_Shared_State(name, callback)
{
    state_Value = new CIF_Bool[size];
    memcpy(state_Value, init_value, sizeof(CIF_Bool) * size);
    state_Count = size;
}

/*
 *  CIF_Shared_State_Bool::~CIF_Shared_State_Bool()
 */
CIF_Shared_State_Bool::~CIF_Shared_State_Bool()
{
    delete state_Value;
}

/*
 *  CIF_Shared_State_Bool::get_State()
 */
CIF_Bool  
CIF_Shared_State_Bool::get_State(
    CIF_Bool                                       dest_state[], 
    int                                         dest_offset, 
    int                                         src_offset, 
    int                                         count)
{
    if(!get_Initialized() || src_offset + count > state_Count)
    {
        return (CIF_Bool)NEXUS_FALSE;
    }

     memcpy(&dest_state[dest_offset], 
            &state_Value[src_offset], 
            count * sizeof(CIF_Bool));
    
    return (CIF_Bool)GLOBUS_TRUE;
}

/*
 *  CIF_Shared_State_Bool::set_State_Changed()
 */
CIF_Bool  
CIF_Shared_State_Bool::set_State_Changed(
    int                                         dest_offset, 
    CIF_Bool                                       src_state[], 
    int                                         src_offset, 
    int                                         count,
    CIF_Shared_State_Changed_Type               type)
{
    CIF_Shared_State_Serial *                   serial;
    CIF_Bool                                    tmp_bool;

    serial = shared_State_Set->new_Shared_State_Serial(this);
        tmp_bool = pack_User_State(serial, 
                                   dest_offset, 
                                   src_state, 
                                   src_offset, 
                                   count);
    if(!tmp_bool)
    {
        return (CIF_Bool)NEXUS_FALSE;
    }

    shared_State_Set->set_State_Changed(serial, type);
    delete serial;

    return (CIF_Bool)GLOBUS_TRUE;
}


/*
 *  CIF_Shared_State_Bool::pack_User_State()
 */
CIF_Bool
CIF_Shared_State_Bool::pack_User_State(
    CIF_Shared_State_Serial *                serial,
    int                                      dest_offset,
    CIF_Bool                                    src_state[],
    int                                      src_offset,
    int                                      count)
{
    int * temp_size = new int[3];

    temp_size[0] = state_Count;
    temp_size[1] = dest_offset;
    temp_size[2] = count;

    if(src_offset + count > state_Count ||
       dest_offset + count > state_Count)
    {
        return (CIF_Bool) GLOBUS_FALSE;
    }
    serial->pack_Int(temp_size, 0, 3);
    serial->pack_Bool(src_state, src_offset, count);

    return (CIF_Bool) GLOBUS_TRUE;
}

/*
 *  CIF_Shared_State_Bool::pack_State()
 */
void   
CIF_Shared_State_Bool::pack_State(
    CIF_Shared_State_Serial *                       serialState)
{
    {
        pack_User_State(serialState, 0, state_Value, 0, state_Count);
    }
}

/*
 *  CIF_Shared_State_Bool::unpack_State()
 */
void   
CIF_Shared_State_Bool::unpack_State(
    CIF_Shared_State_Serial *                    serial)
{
    int temp_val[3];

    serial->unpack_Int(temp_val, 0, 3);

    {
        //should throw an exception
        if(temp_val[0] > state_Count) 
        {
            temp_val[0] = state_Count;
        } 
        if(state_Count < temp_val[1] + temp_val[2])
        {
            temp_val[2] = state_Count - temp_val[1];
        }

        state_Count = temp_val[0] + temp_val[1];
        serial->unpack_Bool(state_Value, temp_val[1], temp_val[2]);
    }
}
    

