#include "CIF_Shared_State_def.H"
#include "nexus.h"
#include "globus_common.h"
#include "globus_nexus.h"
#include "globus_dc.h"

//static int CIF_L_Serial_Message_Size = CIF_CONTROLS_INITIAL_MESSAGE_SIZE;
static int CIF_L_Serial_Message_Size = 10;

////////////////////////////////////////////////////////////
//  CIF_L_Buffer_List_node methods
////////////////////////////////////////////////////////////

/*
 *  CIF_L_Buffer_List_Node::CIF_L_Buffer_List_Node()
 */
CIF_L_Buffer_List_Node::CIF_L_Buffer_List_Node(int size)
{
  local_Size = size;
  local_Buffer = new CIF_Byte[size];
  current_Offset = 0;
  next = NULL;
}

/*
 *  CIF_L_Buffer_List_Node::CIF_L_Buffer_List_Node()
 */
CIF_L_Buffer_List_Node::CIF_L_Buffer_List_Node(CIF_Byte * value, int size)
{
  local_Size = size;
  local_Buffer = new CIF_Byte[size];
  memcpy(local_Buffer, value, size);
  current_Offset = 0;
  next = NULL;
}

/*
 *  CIF_L_Buffer_List_Node::~CIF_L_Buffer_List_Node()
 */
CIF_L_Buffer_List_Node::~CIF_L_Buffer_List_Node()
{
  delete local_Buffer;
}

/*
 *  CIF_L_Buffer_List_Node::get_User_Buffer()
 */
CIF_Byte *
CIF_L_Buffer_List_Node::get_User_Buffer(int size_request)
{
  CIF_Byte * buf_ptr;
  if(current_Offset + size_request > local_Size)
  {
     local_Size = current_Offset;
     return NULL;
  }

  buf_ptr = &local_Buffer[current_Offset];
  current_Offset += size_request;

  return buf_ptr;
}

/*
 *  CIF_L_Buffer_List_Node::get_Buffer()
 */
CIF_Byte *
CIF_L_Buffer_List_Node::get_Buffer()
{
  return local_Buffer;
}

/*
 *  CIF_L_Buffer_List_Node::copy_Buffer()
 */
int
CIF_L_Buffer_List_Node::copy_Buffer(CIF_Byte * buf, 
			            int offset)
{
  memcpy(&buf[offset], local_Buffer, current_Offset);
  return local_Size;
}

////////////////////////////////////////////////////////////
//  CIF_L_Buffer_Manager methods
////////////////////////////////////////////////////////////

/*
 *  CIF_L_Buffer_Manager::CIF_L_Buffer_Manager()
 */
CIF_L_Buffer_Manager::CIF_L_Buffer_Manager(int size)
{
  first = new CIF_L_Buffer_List_Node(size);
  current_Node = first;
  size_Used = 0;
  current_Size = size;
  node_Size = size;
}

/*
 *  CIF_L_Buffer_Manager::CIF_L_Buffer_Manager()
 */
CIF_L_Buffer_Manager::CIF_L_Buffer_Manager(CIF_Byte * value, int size)
{
  first = new CIF_L_Buffer_List_Node(value, size);
  current_Node = first;

  size_Used = size;
  current_Size = size;
  node_Size = size;
}

/*
 *  CIF_L_Buffer_Manager::~CIF_L_Buffer_Manager()
 */
CIF_L_Buffer_Manager::~CIF_L_Buffer_Manager()
{
  CIF_L_Buffer_List_Node * i;
  CIF_L_Buffer_List_Node * j;

  i = first;
  while(i != NULL)
  {
    j = i;
    i = i->next;
    delete j;
  }
}

/*
 *  CIF_L_Buffer_Manager::get_User_Buffer()
 */
CIF_Byte *
CIF_L_Buffer_Manager::get_User_Buffer(int size_requested)
{
  CIF_Byte * buf_ptr;

  buf_ptr = current_Node->get_User_Buffer(size_requested);
  node_Size += size_requested;
  while(buf_ptr == NULL)
  {
    current_Node->next = new CIF_L_Buffer_List_Node(node_Size);
    current_Size += node_Size;
    current_Node = current_Node->next;

    buf_ptr = current_Node->get_User_Buffer(size_requested);

  }
  size_Used += size_requested;

  return buf_ptr;
}

/*
 *  CIF_L_Buffer_Manager::get_Buffer()
 */
CIF_Byte *
CIF_L_Buffer_Manager::get_Buffer()
{
  CIF_L_Buffer_List_Node * i;
  CIF_Byte * buf;
  int offset = 0;
  int len;

  if(current_Node == first)
  {
    return current_Node->get_Buffer();
  }

  buf = new CIF_Byte[size_Used];
  for(i = first; i != NULL; i = i->next)
  {
    len = i->copy_Buffer(buf, offset); 
    offset += len;
  }

  return buf;
}

////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////

/*
 *  CIF_Shared_State_Serial::CIF_Shared_State_Serial()
 */
CIF_Shared_State_Serial::CIF_Shared_State_Serial()
{
  CIF_Byte * buf;

  buffer_Manager = new CIF_L_Buffer_Manager(CIF_L_Serial_Message_Size);
  buf = buffer_Manager->get_User_Buffer(1);
  buf[0] = (CIF_Byte) NEXUS_DC_FORMAT_LOCAL;
  local_Format = NEXUS_DC_FORMAT_LOCAL;
}

/*
 *  CIF_Shared_State_Serial::CIF_Shared_State_Serial()
 */
CIF_Shared_State_Serial::CIF_Shared_State_Serial(CIF_Byte * value, int size)
{
  CIF_Byte * buf;

  buffer_Manager = new CIF_L_Buffer_Manager(value, size);
  buf = buffer_Manager->get_User_Buffer(1);
  local_Format = (int) buf[0];
}

/*
 *  CIF_Shared_State_Serial::~CIF_Shared_State_Serial()
 */
CIF_Shared_State_Serial::~CIF_Shared_State_Serial()
{
  delete buffer_Manager;
}

/*
 *  CIF_Shared_State_Serial::get_Buffer()
 */
CIF_Byte * 
CIF_Shared_State_Serial::get_Buffer()
{
  return buffer_Manager->get_Buffer();
}

/*
 *  CIF_Shared_State_Serial::get_Size_Buffer()
 */
int
CIF_Shared_State_Serial::get_Size_Buffer()
{
  /*  set default size to current size if current size is larger.   */
  if(CIF_L_Serial_Message_Size < buffer_Manager->get_Size())
            CIF_L_Serial_Message_Size = buffer_Manager->get_Size();
  
  return buffer_Manager->get_Size();
}

/*
 *  CIF_Shared_State_Serial::unpack_Byte()
 */
void 
CIF_Shared_State_Serial::unpack_Byte(CIF_Byte value[], 
				     int offset, 
				     int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_byte(count);

  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_get_byte(&buf_ptr, &value[offset], count, local_Format);
}

/*
 *  CIF_Shared_State_Serial::unpack_Char()
 */
void 
CIF_Shared_State_Serial::unpack_Char(char value[], 
				     int offset, 
				     int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_dc_sizeof_remote_char(count, local_Format);

  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_get_char(&buf_ptr, &value[offset], count, local_Format);
}

/*
 *  CIF_Shared_State_Serial::unpack_Bool()
 */
void 
CIF_Shared_State_Serial::unpack_Bool(CIF_Bool value[], 
				     int offset, 
				     int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;
  int byte_array_size;
  CIF_Byte * byte_array;
  int ctr;
  int index;
  int decoded;
  int mask = 1;

  byte_array_size = count / (sizeof(CIF_Byte) * 8) + 1;
  byte_array = new CIF_Byte[byte_array_size];

  size_requested = nexus_sizeof_byte(byte_array_size);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);
  globus_nexus_user_get_byte(&buf_ptr, byte_array, byte_array_size, local_Format);

  index = 0;
  decoded = 0;
  while(decoded < count)
  {
    mask = 1;
    for(ctr = 0; ctr < sizeof(CIF_Byte) * 8 && decoded < count; ctr++)
    {
      if(byte_array[index] & mask)
      {
        value[offset + decoded] = (CIF_Bool)1;
      }
      else
      {
        value[offset + decoded] = (CIF_Bool)0;
      }

      decoded++;   
      mask = mask << 1;
    }
    index++;
  }

}

/*
 *  CIF_Shared_State_Serial::unpack_Short()
 */
void 
CIF_Shared_State_Serial::unpack_Short(short value[], 
				      int offset, 
				      int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_dc_sizeof_remote_short(count, local_Format);

  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_get_short(&buf_ptr, &value[offset], count, local_Format);
}

/*
 *  CIF_Shared_State_Serial::unpack_Int()
 */
void 
CIF_Shared_State_Serial::unpack_Int(int value[], 
				    int offset, 
				    int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_dc_sizeof_remote_int(count, local_Format);

  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_get_int(&buf_ptr, &value[offset], count, local_Format);
}

/*
 *  CIF_Shared_State_Serial::unpack_Long()
 */
void 
CIF_Shared_State_Serial::unpack_Long(long value[], 
				     int offset, 
				     int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_dc_sizeof_remote_long(count, local_Format);

  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_get_long(&buf_ptr, &value[offset], count, local_Format);
}

/*
 *  CIF_Shared_State_Serial::unpack_Float()
 */
void 
CIF_Shared_State_Serial::unpack_Float(float value[], 
				      int offset, 
				      int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_dc_sizeof_remote_float(count, local_Format);

  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_get_float(&buf_ptr, &value[offset], count, local_Format);
}

/*
 *  CIF_Shared_State_Serial::unpack_Double()
 */
void 
CIF_Shared_State_Serial::unpack_Double(double value[], 
				       int offset, 
				       int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_dc_sizeof_remote_double(count, local_Format);

  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_get_double(&buf_ptr, &value[offset], count, local_Format);
}


/*
 *  CIF_Shared_State_Serial::pack_Byte()
 */
void 
CIF_Shared_State_Serial::pack_Byte(CIF_Byte value[], 
				   int offset, 
				   int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_byte(count);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_put_byte(&buf_ptr, &value[offset], count);
}

/*
 *  CIF_Shared_State_Serial::pack_Char()
 */
void 
CIF_Shared_State_Serial::pack_Char(char value[], 
				   int offset, 
				   int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_char(count);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_put_char(&buf_ptr, &value[offset], count);
}

/*
 *  CIF_Shared_State_Serial::pack_Bool()
 */
void 
CIF_Shared_State_Serial::pack_Bool(CIF_Bool value[], 
				   int offset, 
				   int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;
  int byte_array_size;
  CIF_Byte * byte_array;
  int ctr;
  int index;
  int encoded;
  CIF_Byte mask;


  byte_array_size = count / (sizeof(CIF_Byte) * 8) + 1;
  byte_array = new CIF_Byte[byte_array_size];

  index = 0;
  encoded = 0;
  while(encoded < count)
  {
    byte_array[index] = 0;
    mask = 1;
    for(ctr = 0; ctr < sizeof(CIF_Byte) * 8 && encoded < count; ctr++)
    {
      if(value[encoded+offset])
      {
         byte_array[index] = byte_array[index] | mask;
      }
      mask = mask << 1;
      encoded++;   
    }
    index++;
  }

  size_requested = nexus_sizeof_byte(byte_array_size);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);
  globus_nexus_user_put_byte(&buf_ptr, byte_array, byte_array_size);
}

/*
 *  CIF_Shared_State_Serial::pack_Short()
 */
void 
CIF_Shared_State_Serial::pack_Short(short value[], 
				    int offset, 
				    int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_short(count);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_put_short(&buf_ptr, &value[offset], count);
}

/*
 *  CIF_Shared_State_Serial::pack_Int()
 */
void 
CIF_Shared_State_Serial::pack_Int(int value[], 
				  int offset, 
				  int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_int(count);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_put_int(&buf_ptr, &value[offset], count);
}

/*
 *  CIF_Shared_State_Serial::pack_Long()
 */
void 
CIF_Shared_State_Serial::pack_Long(long value[], 
				   int offset, 
				   int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_long(count);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_put_long(&buf_ptr, &value[offset], count);
}

/*
 *  CIF_Shared_State_Serial::pack_Float()
 */
void 
CIF_Shared_State_Serial::pack_Float(float value[], 
				    int offset, 
				    int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_float(count);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_put_float(&buf_ptr, &value[offset], count);
}

/*
 *  CIF_Shared_State_Serial::pack_Double()
 */
void 
CIF_Shared_State_Serial::pack_Double(double value[], 
				     int offset, 
				     int count)
{
  CIF_Byte * buf_ptr;
  int size_requested;

  size_requested = nexus_sizeof_double(count);
  buf_ptr = buffer_Manager->get_User_Buffer(size_requested);

  globus_nexus_user_put_double(&buf_ptr, &value[offset], count);
}

