#include "CIF_Shared_State_def.H"
#include <unistd.h>
#include <stdlib.h>

/************************************************************************
*  CIF_Shared_State_Set
*  --------------------
************************************************************************/
/*
 * CIF_Shared_State_Set::CIF_Shared_State_Set()
 */
CIF_Shared_State_Set::CIF_Shared_State_Set(CIF_Messenger_Protocol protocol)
                    :CIF_Shared_State_Base()
{
  // TODO:   set default message_Space_Url

  init();
  if(protocol == CIF_MESSENGER_TYPE_TCP)
  {
     messenger = new CIF_Messenger_TCP(this);
  }
  else if(protocol == CIF_MESSENGER_TYPE_DUMMY)
  {
     messenger = new CIF_Messenger_Dummy(this);
  }
  message_Space_Url = messenger->get_Message_Space_URL();

}

/*
 * CIF_Shared_State_Set::CIF_Shared_State_Set()
 */
CIF_Shared_State_Set::CIF_Shared_State_Set(
    CIF_Messenger_Protocol                       protocol, 
    char *                                       url)
                    :CIF_Shared_State_Base()
{
  message_Space_Url = new char[strlen(url) + 1];
  strcpy(message_Space_Url, url);


  init();
  if(protocol == CIF_MESSENGER_TYPE_TCP)
  {
     DebugPrint("pre messenger");
     messenger = new CIF_Messenger_TCP(this, message_Space_Url);
     DebugPrint("post messenger");
  }
  else if(protocol == CIF_MESSENGER_TYPE_DUMMY)
  {
     messenger = new CIF_Messenger_Dummy(this, message_Space_Url);
  }

}

/*
 * CIF_Shared_State_Set::CIF_Shared_State_Set()
 */
CIF_Shared_State_Set::~CIF_Shared_State_Set()
{
    delete hash_Table;
}

CIF_Shared_State_Set_GUID *
CIF_Shared_State_Set::get_GUID()
{
    return guid;
}

void 
CIF_Shared_State_Set::state_Change_Occurred(
    char *                                      state_name,    
    CIF_Shared_State_Set_GUID *                 g)
{
    int                                         ctr;
    CIF_Shared_State_Set_Event_Listener *       l;

    for(ctr = 0; ctr < listeners->getElementCount(); ctr++)
    {
        l = (CIF_Shared_State_Set_Event_Listener *)listeners->elementAt(ctr);
        l->state_Change_Occurred(state_name, g);
    }
}

void
CIF_Shared_State_Set::set_Joined(
    CIF_Shared_State_Set_GUID *                g)
{
    int                                        ctr;
    CIF_Shared_State_Set_Event_Listener *      l;
  
    for(ctr = 0; ctr < listeners->getElementCount(); ctr++)
    {
        l = (CIF_Shared_State_Set_Event_Listener *)listeners->elementAt(ctr);
        l->set_Joined(g);
    }
}

void 
CIF_Shared_State_Set::set_Exited(CIF_Shared_State_Set_GUID * g)
{
  int ctr;
  CIF_Shared_State_Set_Event_Listener * l;

  for(ctr = 0; ctr < listeners->getElementCount(); ctr++)
  {
    l = (CIF_Shared_State_Set_Event_Listener *)listeners->elementAt(ctr);
    l->set_Exited(g);
  }
}

char * 
CIF_Shared_State_Set::get_Message_Space_URL()
{
    return message_Space_Url;
}

void 
CIF_Shared_State_Set::state_Added(char * state_name,   
				  CIF_Shared_State_Set_GUID * source_set)
{
  int ctr;
  CIF_Shared_State_Set_Event_Listener * l;

  for(ctr = 0; ctr < listeners->getElementCount(); ctr++)
  {
    l = (CIF_Shared_State_Set_Event_Listener *)listeners->elementAt(ctr);
    l->state_Added(state_name, source_set);
  }
}

void
CIF_Shared_State_Set::close()
{
   messenger->close();
}

CIF_Bool
CIF_Shared_State_Set::poll()
{
  return messenger->poll();
}

/*
 * CIF_Shared_State_Set::add_Set_Event_Listener()
 */
void
CIF_Shared_State_Set::add_Set_Event_Listener(CIF_Shared_State_Set_Event_Listener * l)
{
   listeners->addElement((void *) l);
}

/*
 * CIF_Shared_State_Set::remove_Set_Event_Listener()
 */
void
CIF_Shared_State_Set::remove_Set_Event_Listener(CIF_Shared_State_Set_Event_Listener * l)
{
   listeners->removeElement((void *) l);
}

/*
 * CIF_Shared_State_Set::init()
 */
void
CIF_Shared_State_Set::init()
{
    CIF_Byte *                             id;
    int                                    id_len;
    long                                   pid;
    long                                   hid;
    long                                   t;
    char                                   shid[100];
    int                                    slen=90;


    globus_mutex_init(&mutex, GLOBUS_NULL);
    globus_cond_init(&cond, GLOBUS_NULL);
    globus_libc_gethostname(shid, slen);
    slen = strlen(shid);
    id_len = sizeof(long) * 2 + slen;
    id = new CIF_Byte[id_len];

    t = clock();
    pid = (long)getpid();

    memcpy(id, shid, slen);
    memcpy(&id[slen], &pid, sizeof(long));
    memcpy(&id[sizeof(long)+slen], &t, sizeof(long));

    listeners = new CIF_I_List();

    hash_Table = new CIF_I_Hash_Table(CIF_SHARED_STATE_HASH_TABLE_SIZE);
    guid = new CIF_Shared_State_Set_GUID(id, id_len);

    delete id;
}

/*
 * CIF_Shared_State_Set::add_Shared_State()
 */
void   
CIF_Shared_State_Set::add_Shared_State(
    CIF_Shared_State * shared_state)
{
//    lock();
    {
        hash_Table->putElement(
	    shared_state->get_Name(), 
	    (void *) shared_state);
        shared_state->joined_Set(this);
        messenger->add_Shared_State(shared_state);
    }
//    unlock();

    shared_state->lock();
    {
        while(!shared_state->get_Initialized())
	{
            shared_state->wait();
	}
    }
    shared_state->unlock();
}

/*
 * CIF_Shared_State_Set::getSharedState()
 */
CIF_Shared_State * 
CIF_Shared_State_Set::get_Shared_State(char * shared_state_name)
{
    CIF_Shared_State *                    return_state;

    lock();
    {
        return_state = (CIF_Shared_State *) 
  		           hash_Table->getElement(shared_state_name);
    }
    unlock();

    return  return_state;

}

/*
 * CIF_Shared_State_Set::getSharedState()
 */
CIF_Shared_State **
CIF_Shared_State_Set::get_Shared_States()
{
    CIF_Shared_State **                   return_states;

    lock();
    {
        return_states = (CIF_Shared_State **)
            hash_Table->getEnumeration();
    }
    unlock();
    
    return return_states;
}

int
CIF_Shared_State_Set::get_Shared_State_Count()
{
    int                                      rc;

    lock();
    {
        rc = hash_Table->getElementCount();
    }
    unlock();

    return rc;
}

/*
 * new_Shared_State_Serial()
 */
CIF_Shared_State_Serial * 
CIF_Shared_State_Set::new_Shared_State_Serial(
    CIF_Shared_State *                             state) 
{
    CIF_Shared_State_Serial * serial;

    serial = messenger->new_Shared_State_Serial();

    pack_User_Set(serial, state);

    return serial;
}

void
CIF_Shared_State_Set::pack_User_Set(
    CIF_Shared_State_Serial *                      serial, 
    CIF_Shared_State *                             state)
{
    char *                                         state_name;
    int                                            len[1];
 
    lock();
    {
        len[0] = guid->get_Length();
        serial->pack_Int(len, 0, 1);
        serial->pack_Byte(guid->get_Bytes(), 0, len[0]);

        state_name = state->get_Name();
        len[0] = strlen(state_name);
        serial->pack_Int(len, 0, 1);
        serial->pack_Char(state_name, 0, len[0]);
    }
    unlock();
}

/*
 * set_State_Changed()
 */
void 
CIF_Shared_State_Set::set_State_Changed(
    CIF_Shared_State_Serial *                         serial,
    CIF_Shared_State_Changed_Type                     type)
{
    messenger->send(serial, type);
}

/*
 * receive()
 */
void
CIF_Shared_State_Set::receive(
    CIF_Shared_State_Serial *                         serial,
    CIF_Shared_State_Changed_Type                     type,
    long                                              seq_num)
{
    char *                                            state_name;
    int                                               len[1];
    CIF_Shared_State *                                state;
    CIF_Byte *                                        guid_bytes;
    CIF_Shared_State_Set_GUID *                       action_guid;

    serial->unpack_Int(len, 0, 1);
    guid_bytes = new CIF_Byte[len[0]];
    serial->unpack_Byte(guid_bytes, 0, len[0]);
    action_guid = new CIF_Shared_State_Set_GUID(guid_bytes, len[0]);

    serial->unpack_Int(len, 0, 1);
    state_name = new char[len[0]+1];
    serial->unpack_Char(state_name, 0, len[0]);
    state_name[len[0]] = '\0';
  
//    lock();
    {
        state = (CIF_Shared_State *) hash_Table->getElement(state_name);
    }
//    unlock();

    if(state != NULL) 
    {
	if(type == INITIALIZE)
	{
            state->set_Initialized(GLOBUS_TRUE);
	    state->state_Changed(serial, type, action_guid);
	}
	else if(type == NAK_INITIALIZE)
	{
            state->set_Initialized(GLOBUS_TRUE);
	}
	else if(state->get_Initialized())
	{
	    state->state_Changed(serial, type, action_guid);
	}
    }

    state_Change_Occurred(state_name, action_guid);

    delete state_name;
    delete guid_bytes;
}
