/******************************************************************************
CIF_Toolkit_Object.C

Description:

  A generic CIF object.  Eventually, this will be used to identify objects and
  object hierarchies.  This will be accomplished by having each a UUID
  associated with each object definition and then requiring that the object
  constructors add their associated UUID to an ID list.

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/cpp/libraries/toolkit/CIF_Toolkit_Object.C,v $
  $Date: 1998/09/28 20:16:38 $
  $Revision: 1.2 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

#include "CIF_Toolkit_Object.H"


CIF_Toolkit_Object::CIF_Toolkit_Object()
{
    member_list = GLOBUS_NULL;
}

CIF_Toolkit_Object::~CIF_Toolkit_Object()
{
    globus_list_free(member_list);
}

CIF_Bool
CIF_Toolkit_Object::is_Member(
    ARG_IN const CIF_Toolkit_UUID &		object_uuid)
{
    if (globus_list_search_pred(member_list,
				compare_Members,
				(void *) &object_uuid) != GLOBUS_NULL)
    {
	return true;
    }
    else
    {
	return false;
    }
}

void
CIF_Toolkit_Object::add_Member(
    ARG_IN const CIF_Toolkit_UUID *		object_uuid)
{
    globus_list_insert(&member_list, (void *) object_uuid);
}

int
CIF_Toolkit_Object::compare_Members(
    void *					uuid_ptr1,
    void *					uuid_ptr2)
{
    const CIF_Toolkit_UUID & uuid1 = * (CIF_Toolkit_UUID *) uuid_ptr1;
    const CIF_Toolkit_UUID & uuid2 = * (CIF_Toolkit_UUID *) uuid_ptr2;

    if (uuid1 == uuid2)
    {
	return 1;
    }
    else
    {
	return 0;
    }
}
