AC_DEFUN(LAC_UTILS_INIT,
[
    case ${build_os} in
	cygwin32* | winnt*)
	    PATH_SEP=";"
	    SLASH_BACKWARD='\\'
	    SLASH_FORWARD='/'
	    DEV_NULL='NUL'
	    ;;
	*)
	    PATH_SEP=":"
	    SLASH_BACKWARD='/'
	    SLASH_FORWARD='/'
	    DEV_NULL='/dev/null'
	    ;;
    esac
    AC_SUBST(PATH_SEP)
    AC_SUBST(SLASH_BACKWARD)
    AC_SUBST(SLASH_FORWARD)
    AC_SUBST(DEV_NULL)

    changequote(<<,>>)

    dnl bs_path(path)
    dnl
    dnl Take a UNIX style path and if necessary convert it to a Windows
    dnl compatible path
    dnl
    bs_path() {
    	src_path="$<<1>>"
	src_path=`echo "${src_path}" | sed "s-/-${SLASH_BACKWARD}-g"`
	src_path=`echo "${src_path}" | sed 's/[\]/\\\\\\\\/g'`
    	echo "$src_path"
    }

    dnl resolve_path(path, rel_path)
    dnl
    dnl Take a relative or absolute path and adjust it according to a relative
    dnl path of the form ./../.../..
    dnl
    resolve_path() {
    	src_path="$<<1>>"
    	rel_path="$<<2>>"
    	while test "${rel_path}" != "." ; do
    	    rel_path=`echo "${rel_path}" | sed 's-/..--'`
    	    src_path=`echo "${src_path}" | sed 's-/[^/]*$--'`
    	done
    	echo "$src_path"
    }

    dnl make_relative_path(abs_path, rel_path)
    dnl
    dnl Take the supplied absolute path and return a relative path
    dnl based on the depth of the supplied relative path.
    dnl
    make_relative_path() {
    	src_path="$<<1>>"
    	rel_path="$<<2>>"
	new_path=""
    	while test "${rel_path}" != "." ; do
    	    rel_path=`echo "${rel_path}" | sed 's-/..--'`
            new_path="/`echo "${src_path}" | sed 's-.*/--'`${new_path}"
    	    src_path=`echo "${src_path}" | sed 's-/[^/]*$--'`
    	done
    	echo ".$new_path"
    }

    changequote([,])
])

AC_DEFUN(LAC_UTILS_PATHS,
[
    dnl
    dnl Compute and publish source and build directory paths
    dnl
    srcdir_fs=${srcdir}
    topsrcdir_fs=`resolve_path ${srcdir_fs} REL_TOP_DIR`
    configdir_fs=${topsrcdir_fs}/LAC_DIR_NAME
    builddir_fs=.
    topbuilddir_fs=REL_TOP_DIR
    absbuilddir_fs=`pwd`
    relbuilddir_fs=`make_relative_path ${absbuilddir_fs} REL_TOP_DIR`
    abstopbuilddir_fs=`resolve_path ${absbuilddir_fs} REL_TOP_DIR`
    if test "${prefix}" = "NONE" ; then
        prefix="${abstopbuilddir_fs}${SLASH_FORWARD}install"
    fi
    prefix_fs=${prefix}

    AC_SUBST(prefix_fs)
    AC_SUBST(srcdir_fs)
    AC_SUBST(topsrcdir_fs)
    AC_SUBST(configdir_fs)
    AC_SUBST(builddir_fs)
    AC_SUBST(topbuilddir_fs)
    AC_SUBST(relbuilddir_fs)

    echo "prefix_fs=${prefix_fs}"
    echo "srcdir_fs=${srcdir_fs}"
    echo "topsrcdir_fs=${topsrcdir_fs}"
    echo "configdir_fs=${configdir_fs}"
    echo "builddir_fs=${builddir_fs}"
    echo "topbuilddir_fs=${topbuilddir_fs}"
    echo "relbuilddir_fs=${relbuilddir_fs}"

    prefix_bs=`bs_path ${prefix_fs}`
    srcdir_bs=`bs_path ${srcdir_fs}`
    topsrcdir_bs=`bs_path ${topsrcdir_fs}`
    configdir_bs=`bs_path ${configdir_fs}`
    builddir_bs=`bs_path ${builddir_fs}`
    topbuilddir_bs=`bs_path ${topbuilddir_fs}`
    relbuilddir_bs=`bs_path ${relbuilddir_fs}`

    AC_SUBST(prefix_bs)
    AC_SUBST(srcdir_bs)
    AC_SUBST(topsrcdir_bs)
    AC_SUBST(configdir_bs)
    AC_SUBST(builddir_bs)
    AC_SUBST(topbuilddir_bs)
    AC_SUBST(relbuilddir_bs)

    echo "prefix_bs=${prefix_bs}"
    echo "srcdir_bs=${srcdir_bs}"
    echo "topsrcdir_bs=${topsrcdir_bs}"
    echo "configdir_bs=${configdir_bs}"
    echo "builddir_bs=${builddir_bs}"
    echo "topbuilddir_bs=${topbuilddir_bs}"
    echo "relbuilddir_bs=${relbuilddir_bs}"

    dnl
    dnl Determine and publish paths to common utilities
    dnl
    dnl AC_PROG_AWK
    dnl AC_PATH_PROG(CAT, cat)
    dnl AC_PATH_PROG(CP, cp)
    dnl AC_PATH_PROG(ECHO, echo)
    dnl AC_PATH_PROG(PRINTF, printf, echo)
    dnl AC_PATH_PROG(GREP, grep)
    AC_PATH_PROG(LN, ln)
    AC_PROG_LN_S
    AC_PROG_MAKE_SET
    AC_PATH_PROG(MKDIR, mkdir)
    dnl AC_PATH_PROG(MV, mv)
    dnl AC_PATH_PROG(PWD, pwd)
    AC_PATH_PROG(RM, rm)
    AC_PATH_PROG(RMDIR, rmdir)
    dnl AC_PATH_PROG(SED, sed)
    dnl AC_PATH_PROG(TOUCH, touch)
    AC_PATH_PROG(TRUE, true)
])
