/******************************************************************************
Simple_Example.java

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/java/examples/cif/comm/Simple_Example.java,v $
  $Date: 1998/10/23 19:46:52 $
  $Revision: 1.3 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

import cif.comm.*;
import cif.toolkit.*;


/**
 * A simple example cif.comm program to test and demonstrate the Java
 * implementation.  This program demonstrates the base set of cif.comm classes
 * and methods necessary to communicate data.  More advanced features can be
 * found in the other example programs.
 */
public
class Simple_Example
{
    /*
     * usage()
     */
    static
    void
    usage()
    {
	System.err.println(
	    "\nusage: Simple_Example [-c <nmsgs>] [-f] [-l] [-t <timeout>] " +
	    "<-s|-r> <URL>\n");
	System.err.println("    -c\tmessage count");
	System.err.println("    -f\trepeat forever");
	System.err.println("    -l\tlistener");
	System.err.println("    -r\treciever");
	System.err.println("    -s\tsender");
	System.err.println("    -t\ttimeout value\n");

	System.exit(1);
    }

    /*
     * main()
     */
    public static final
    void    
    main(
	String []				args)
    throws
	Exception
    {
	final int				ARGC_REQ = 1;

	int					argc;
	int					argn;
	int					msg_cnt = 1;
	boolean					forever = false;
	boolean					listener = false;
	boolean					sender = false;
	boolean					receiver = false;
	long					timeout = Timeout.INFINITY;    
	String					url = null;

	/*
	 * Process arguments
	 */
	argc = args.length;
	argn = -1;
	while(++argn < argc - ARGC_REQ)
	{
	    if (args[argn].compareTo("-c") == 0 &&
		++argn < argc - ARGC_REQ)
	    {
		msg_cnt = Integer.parseInt(args[argn]);
		continue;
	    }
	    else if (args[argn].compareTo("-f") == 0)
	    {
		forever = true;
		continue;
	    }
	    else if (args[argn].compareTo( "-l") == 0)
	    {
		listener = true;
		continue;
	    }
	    else if (args[argn].compareTo("-s") == 0)
	    {
		sender = true;
		continue;
	    }
	    else if (args[argn].compareTo("-r") == 0)
	    {
		receiver = true;
		continue;
	    }
	    else if (args[argn].compareTo("-t") == 0 &&
		     ++argn < argc - ARGC_REQ)
	    {
		timeout = Long.parseLong(args[argn]);
		continue;
	    }
	    else
	    {
		System.err.println("\nunknown argument: " + args[argn] + "\n");
		usage();
	    }
	}

	/*
	 * Verify that all of the necessary arguments were specified
	 */
	if (argn + ARGC_REQ == argc)
	{
	    url = new String(args[argn]);
	}
	else
	{
	    usage();
	}

	if (!sender && !receiver)
	{
	    usage();
	}
    
	try
	{
	    /*
	     * Activate cif.comm
	     *
	     * cif.comm.Comm.activate() must be executed before any other
	     * cif.comm classes are instantiated or methods are called.  The
	     * activation methods contain reference counting, so each module
	     * which uses one of the CIF libraries can safely envoke the
	     * libraries activation method without worrying about the potential
	     * problems associated with reactivation or early deactivation.
	     * However, it is important that all modules which perform
	     * activation also perform deactivation when the have finished
	     * using the CIF library.  Failing to perform deactivation can have
	     * unpredicable results for some libraries.
	     */
	    Comm.activate();

	    /*
	     * Establish a listener object and the wait for an incoming
	     * connection
	     */
	    Listener				cif_listener = null;

	    if (listener)
	    {
		System.out.println(
		    "Establishing listener object for URL=" + url);

		cif_listener = Factory.get_Listener(url);
		System.out.println("get_Listener(): completed");
	    
		System.out.println(
		    "\nListening at " + cif_listener.get_URL() + "\n");
	    }
    

	    do
	    {
		Connection				cif_conn = null;

		if (listener)
		{
		    /*
		     * Wait for a remote process contact the listener and form
		     * a new connection.
		     */
		    System.out.println("Waiting for connection");

		    if (timeout != Timeout.INFINITY)
		    {
			cif_conn = cif_listener.get_Connection(timeout);
		    }
		    else
		    {
			cif_conn = cif_listener.get_Connection();
		    }
	    
		    System.out.println("get_Connection(): completed");
		}
		else
		{
		    /*
		     * Attempt to establish a new connection
		     */
		    System.out.println(
			"Establishing connection object for URL=" + url);

		    if (timeout != Timeout.INFINITY)
		    {
			cif_conn = Factory.get_Connection(url, timeout);
		    }
		    else
		    {
			cif_conn = Factory.get_Connection(url);
		    }

		    System.out.println("get_Connection(): completed");
		}
		System.out.println("Connection established.");
	
		int					cnt;

		if (sender)
		{
		    /*
		     * Send msg_cnt number of messages
		     */
		    System.out.println("Sending " + msg_cnt + "messages");
	    
		    for (cnt = 0; cnt < msg_cnt; cnt++)
		    {
			String			msg_string;
			byte []			msg_buf;

			msg_string = new
			    String("data for message " + cnt + "\0");
			msg_buf = msg_string.getBytes();
		    
			cif_conn.send(msg_buf, msg_buf.length, cnt);

			System.out.println("send(): completed");
		    }
		}
		else
		{
		    /*
		     * Wait for the first message to be received.  This is not
		     * necessary since receive will block until a message is
		     * receive; however, it demostrates the use of the poll()
		     * method.
		     */
		    while (cif_conn.poll() == false)
		    {
			Thread.yield();
		    }

		    /*
		     * Revceive msg_cnt number of messages
		     */
		    for (cnt = 0; cnt < msg_cnt; cnt++)
		    {
			Message			message;
	    
			if (timeout != Timeout.INFINITY)
			{
			    message = cif_conn.receive(timeout);
			}
			else
			{
			    message = cif_conn.receive();
			}
	    

			System.out.println("receive(): completed");

			System.out.println(
			    "message received" +
			    ", size=" + message.get_Info().size +
			    ", tag=" + message.get_Info().tag +
			    ", data=" + new String(message.get_Data()));
		    }
		}

		/*
		 * Close the connection and delete the connection object
		 */
		System.out.println(
		    "Communication complete.  Closing connection.");

		cif_conn.close();

		System.out.println("close(): completed");
	    }
	    while (forever);

	    /*
	     * Close the listener and delete the listener object
	     */
	    if (listener)
	    {
		System.out.println("Closing listener.");

		cif_listener.close();

		System.out.println("close(): completed");
	    }
	}
	finally
	{
	    /*
	     * Detactivate CIF
	     *
	     * cif.comm.Comm.deactivate() must be called when the application
	     * has finished using CIF Comm.  Normally, this is done right
	     * before the application exits.
	     */
	    Comm.deactivate();
	}
    }
}

