/******************************************************************************
Comm.java

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/java/libraries/cif/comm/Comm.java,v $
  $Date: 1999/02/22 15:31:26 $
  $Revision: 1.4 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

package cif.comm;

import nexus.*;


/**
 * An static initialization and generic operations class for CIF_Comm.
 * cif.comm.Comm.activate() must be called before any other object in the
 * CIF_Comm pacakage may be used.  Similarily, cif.comm.Comm.deactivate() must
 * be called before the process terminates.  See the appropriate member
 * descriptions for details.
 */
public
class Comm
{
    public static final String TCP_URL_SCHEME = "x-cif-tcp";
    public static final String UDP_URL_SCHEME = "x-cif-udp";
    public static final String UUM_URL_SCHEME = "x-cif-uum";
    public static final String ROM_URL_SCHEME = "x-cif-rom";
    
    /**
     * Intialize internal state and packages used by CIF_Comm.  This method
     * must be called before any other object in the CIF_Comm package is used.
     * Not doing so may result in undesireable and unpredictable package
     * failures.
     *
     * Activation and deactivation are reference counted, and therefore may be
     * called multiple times so long as the number of deactivations equals the
     * number of activation before the program is terminated.
     */
    public static synchronized
    boolean
    activate()
    {
	if (l_ref_cnt == 0)
	{
	    l_nexus = new Nexus();
	    l_nexus.activate();
	}

	try
	{
	    acquire_Nexus();
	}
	catch (Comm_Inactive_Exception e)
	{
	    return false;
	}

	return true;
    }


    /**
     * Properly shutdown CIF_Comm and any packages used by it.  This method
     * must an equal number of time to the activate() method before the process
     * is terminated.  Failing to do may result in buffered messages be
     * discarded, etc.
     */
    public static synchronized
    boolean
    deactivate()
    {
	release_Nexus();
	
	return true;
    }

    static synchronized
    Nexus
    acquire_Nexus()
    throws
	Comm_Inactive_Exception
    {
	if (l_nexus == null)
	{
	    throw new Comm_Inactive_Exception(
		"cif.comm.Comm:acquire_Nexus(): Comm has not be activated!");
	}

	l_ref_cnt++;

	return l_nexus;
    }

    static synchronized
    void
    release_Nexus()
    {
	l_ref_cnt--;

	if (l_ref_cnt == 0)
	{
	    l_nexus.deactivate();
	    l_nexus = null;
	}
    }
    

    private static int				l_ref_cnt = 0;
    private static Nexus			l_nexus = null;
}
