/******************************************************************************
Connection.java

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/java/libraries/cif/comm/Factory.java,v $
  $Date: 1999/03/02 01:11:38 $
  $Revision: 1.7 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

package cif.comm;

import cif.toolkit.*;


/**
 * A group of static methods used to construct CIF connection and 
 * Listener objects.
 *  
 */
public class Factory
{

    /**
    *
    *  Get an interface to a connection object using the protocol type given in
    *  the url.
    *  
    *   @see cif.comm.Factory#get_Connection(String, Connection_Properies, Timeout);
    */
    public static
    Connection
    get_Connection(
	String					url)
    throws
	Object_Active_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	try
	{
	    return l_get_Connection(url,
				    new Connection_Properties(),
				    new Timeout(Timeout.INFINITY));
	}
	catch (Timeout_Expired_Exception e)
	{
	    throw new Internal_Error(
		"Factory.get_Connection() caught an unexpected " +
		"Timeout_Expired_Exception");
	}
    }
    
    /**
    *
    *  Get an interface to a connection object using the protocol type given in
    *  the url.
    *  
    *   @param timeout
    *      long integer representing the amount of time to wait for a successful
    *      connection.
    *
    *   @see cif.comm.Factory#get_Connection(String, Connection_Properies, Timeout);
    */
    public static
    Connection
    get_Connection(
	String					url,
	long					timeout)
    throws
	Object_Active_Exception,
	Timeout_Expired_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	return l_get_Connection(url,
				new Connection_Properties(),
				new Timeout(timeout));
    }

    /**
    *
    *  Get an interface to a connection object using the protocol type given in
    *  the url.
    *  
    *   @param timeout
    *      a cif.toolkit.Timeout object representing the amount of time to wait 
    *      for a successful connection.
    *  
    *   @see cif.comm.Factory#get_Connection(String, Connection_Properies, Timeout);
    */
    public static
    Connection
    get_Connection(
	String					url,
	Timeout					timeout)
    throws
	Object_Active_Exception,
	Timeout_Expired_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	return l_get_Connection(url,
				new Connection_Properties(),
				timeout);
    }

    /**
    *
    *  Get an interface to a connection object using the protocol type given in
    *  the url.
    *  
    *   @see cif.comm.Factory#get_Connection(String, Connection_Properies, Timeout);
    */
    public static
    Connection
    get_Connection(
	String					url,
	Connection_Properties			properties)
    throws
	Object_Active_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	try
	{
	    return l_get_Connection(url,
				    properties,
				    new Timeout(Timeout.INFINITY));
	}
	catch (Timeout_Expired_Exception e)
	{
	    throw new Internal_Error(
		"Factory.get_Connection() caught an unexpected " +
		"Timeout_Expired_Exception");
	}
    }
    
    /**
    *
    *  Get an interface to a connection object using the protocol type given in
    *  the url.
    *  
    *   @see cif.comm.Factory#get_Connection(String, Connection_Properies, Timeout);
    */
    public static
    Connection
    get_Connection(
	String					url,
	Connection_Properties			properties,
	long					timeout)
    throws
	Object_Active_Exception,
	Timeout_Expired_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	return l_get_Connection(url,
				properties,
				new Timeout(timeout));
    }

    /**
    *  Get an interface to a connection object using the protocol type given in
    *  the url.
    *
    *  @param url
    *      
    *      a string of the following format:
    *        <BLOCKQUOTE><P>
    *           [protocol]://[hostname]:[port]/[parameters]
    *        </P></BLOCKQUOTE>
    *      ex:
    *        <BLOCKQUOTE><P>
    *            x-cif-uum://228.225.225.225:32498/ttl=10+loopback=true
    *        </P><BLOCKQUOTE>
    *        <BR>
    *        The above string requests a ureliable unordered multicast
    *        connection to IP address 228.255.255.255 on port 32498 with
    *        a time to live of 10 and setting loopback to true.
    *<BR><BR>
    *      protocols strings supported by cif.comm:
    *      <UL>
    *        <LI>x-cif-tcp:
    *            a tcp connection</LI>
    *        <LI>x-cif-udp:
    *            a udp connection</LI>
    *        <LI>x-cif-uum:
    *            an unreliable unordered multicast connection</LI>
    *        <LI>x-cif-rom:
    *            a reliable unordered multicast connection</LI>
    *      </UL>
    *<BR>
    *  @param properties
    *      This parameters allow the user to set up a connection with
    *      a cif.toolkit.Event_Listener
    *<BR>
    *  @param timeout
    *      a cif.toolkit.Timeout object representing the amount of 
    *      time to wait for a successful connection.
    *  
    *
    */
    public static
    Connection
    get_Connection(
	String					url,
	Connection_Properties			properties,
	Timeout					timeout)
    throws
	Object_Active_Exception,
	Timeout_Expired_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	return l_get_Connection(url,
				properties,
				timeout);
    }

    /**
    *  Creates a listener on a given url.  For url format see get_Connection().
    *   
    *   @see cif.comm.Factory#get_Connection(String, Connection_Properies, Timeout);
    */
    public static
    Listener
    get_Listener(
	String					url)
    throws
	Object_Active_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception
    {
	return l_get_Listener(url, new Listener_Properties());
    }
    
    public static
    Listener
    get_Listener(
	String					url,
	Listener_Properties			properties)
    throws
	Object_Active_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception
    {
	return l_get_Listener(url, properties);
    }
    
    private static
    Connection
    l_get_Connection(
	String					url_string,
	Connection_Properties			properties,
	Timeout					timeout)
    throws
	Object_Active_Exception,
	Timeout_Expired_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	URL					url;
	I_Connection				conn;
	
	
	url = new URL(url_string);

	conn = null;
	
	if (url.get_Protocol().compareTo(Comm.TCP_URL_SCHEME) == 0)
	{
	    conn = new I_Connection_TCP();
	}
	else if(url.get_Protocol().compareTo(Comm.UDP_URL_SCHEME) == 0)
	{
	    conn = new I_Connection_UDP();
	}
	else if(url.get_Protocol().compareTo(Comm.UUM_URL_SCHEME) == 0)
	{
	    conn = new I_Connection_UUM();
	}
	else if(url.get_Protocol().compareTo(Comm.ROM_URL_SCHEME) == 0)
	{
	    conn = new I_Connection_ROM();
	}
	else
	{
	    throw new Bad_Parameter_Exception(
		1, "unrecognized protocol in URL");
	}

	conn.activate(url, properties, timeout);
		      
	return conn;
    }

    private static
    Listener
    l_get_Listener(
	String					url_string,
	Listener_Properties			properties)
    throws
	Object_Active_Exception,
	Bad_Parameter_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception
    {
	URL					url;
	I_Listener				listener;
	
	
	url = new URL(url_string);

	listener = null;
	
	if (url.get_Protocol().compareTo(Comm.TCP_URL_SCHEME) == 0)
	{
	    listener = new I_Listener_TCP();
	}
	else if (url.get_Protocol().compareTo(Comm.UDP_URL_SCHEME) == 0)
	{
	    listener = new I_Listener_UDP();
	}
	else if (url.get_Protocol().compareTo(Comm.UUM_URL_SCHEME) == 0)
	{
	    listener = new I_Listener_UUM();
	}
	else if (url.get_Protocol().compareTo(Comm.ROM_URL_SCHEME) == 0)
	{
	    listener = new I_Listener_ROM();
	}
	else
	{
	    throw new Bad_Parameter_Exception(
		1, "unrecognized protocol in URL");
	}

	listener.activate(url, properties);
		      
	return listener;
    }
}
