/******************************************************************************
I_Connection_ROM.java

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/java/libraries/cif/comm/I_Connection_ROM.java,v $
  $Date: 1999/02/25 21:39:13 $
  $Revision: 1.2 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

package cif.comm;

import nexus.*;
import cif.toolkit.*;

/**
 * This class completes the Connection implementation for the ROM protocol.
 */
class I_Connection_ROM
extends I_Connection
{
    protected
    void
    open_Connection(
	URL					url,
	I_Endpoint_Properties			ep_properties,
	Timeout					timeout)
    throws
	Bad_Parameter_Exception,
	Timeout_Expired_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
       /*
	* Verify URL information
        */
        if (url.get_Address() == null)
        {
           throw new Bad_Parameter_Exception(
               1, "process group id must be specified in a connection URL");
        }
        if(url.get_Port() >= 1)
	{
           throw new Bad_Parameter_Exception(
               1, "a port number must not be specified in a connection URL");
	}

	if (url.get_Protocol().compareTo(Comm.ROM_URL_SCHEME) != 0)
	{
	    throw new Bad_Parameter_Exception(
		1, "expected URL Scheme to be " + Comm.ROM_URL_SCHEME);
	}

	// get options
        String                parameters = url.get_Address(); 
        int                   pgid;
	if(parameters != null && !parameters.equals("/"))
	{
	   try
	   {
	      String s = parameters.substring(0);
              pgid = new Integer(s).intValue();
	   }
	   catch(Exception e)
	   {
	       throw new Bad_Parameter_Exception(
	   	   1, "expected URL Scheme to be " + Comm.ROM_URL_SCHEME);
	   }
	}
        else
	{
	    throw new Bad_Parameter_Exception(
		1, "expected a process group id " + Comm.ROM_URL_SCHEME);
	}

	System.out.println("Process group id = " + pgid);
        TotemInfo             totem_info = new TotemInfo(pgid);

        try
	{
	  ep_properties.set_Protocol(totem_info);

          i_ep = new I_Endpoint(); 
	  i_ep.activate(ep_properties);

          i_sp = new I_Startpoint();
          i_sp.activate(i_ep);
        }
	catch (Bad_Parameter_Exception e)
	{
	  throw new Internal_Error(
				   "cif.comm.I_Connection_ROM.open_Connection(): " +
	  		            "caught an unexpected Bad_Parameter_Exception");
	}
        catch (Object_Active_Exception e)
	{
	    throw new Internal_Error(
			    "cif.comm.I_Connection_ROM.open_Connection(): " +
			    "caught unexpected Object_Active_Exception");
        }

	i_flow = I_FLOW_BIDIRECTIONAL;
    }
    
	    
    
    protected
    void
    open_Connection(
	GetBuffer			nx_get_buf,
	I_Endpoint_Properties		ep_properties,
	Timeout				timeout)
    throws
	Timeout_Expired_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	  throw new Internal_Error(
				   "cif.comm.I_Connection_ROM.open_Connection(): " +
	  		            "caught an unexpected Bad_Parameter_Exception");
    }
}


