/******************************************************************************
I_Connection_UDP.java

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/java/libraries/cif/comm/I_Connection_UDP.java,v $
  $Date: 1999/02/25 21:39:14 $
  $Revision: 1.2 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

package cif.comm;

import nexus.*;
import cif.toolkit.*;

/**
 * This class completes the Connection implementation for the UDP protocol.
 */
class I_Connection_UDP
extends I_Connection
{
    protected
    void
    open_Connection(
	URL					url,
	I_Endpoint_Properties			ep_properties,
	Timeout					timeout)
    throws
	Bad_Parameter_Exception,
	Timeout_Expired_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {

       /*
        * Verify URL information
        */
	if (url.get_Hostname() == null)
	{
	    throw new Bad_Parameter_Exception(
	        1, "hostname must be specified in a connection URL");
	}

	if (url.get_Port() < 1)
	{
	  throw new Bad_Parameter_Exception(
	     1, "port number may not be zero in a connection URL");
	}

	if (url.get_Protocol().compareTo(Comm.UDP_URL_SCHEME) != 0)
	{
	    throw new Bad_Parameter_Exception(
		1, "expected URL Scheme to be " + Comm.UDP_URL_SCHEME);
	}

	I_Startpoint				rctrl_sp;
	I_Startpoint				lctrl_sp;
	I_Startpoint				lcomm_sp;
	I_Startpoint				rcomm_sp;
	I_Endpoint				lctrl_ep;
	I_Endpoint				lcomm_ep;

	ep_properties.set_Protocol(new UdpInfo());
	try
	{
	    PutBuffer				nx_put_buf;
	    GetBuffer				nx_get_buf;

	    try
	    {
		rctrl_sp = new I_Startpoint();
		rctrl_sp.activate(url.get_Hostname(), url.get_Port());
	
		lctrl_ep = new I_Endpoint();
		lctrl_ep.activate(new I_Endpoint_Properties());

		lctrl_sp = new I_Startpoint();
		lctrl_sp.activate(lctrl_ep);

		lcomm_ep = new I_Endpoint();
		lcomm_ep.activate(ep_properties);
	
		lcomm_sp = new I_Startpoint();
		lcomm_sp.activate(lcomm_ep);
	    }
	    catch (Bad_Parameter_Exception e)
	    {
		throw new Internal_Error(
		    "cif.comm.I_Connection_UDP.open_Connection(): " +
		    "caught an unexpected Bad_Parameter_Exception");
	    }
	    
	    try
	    {
		Nexus				nx;

		nx = Comm.acquire_Nexus();
		nx_put_buf =
		    nx.buffer_init(lctrl_sp.sizeof() + lcomm_sp.sizeof(), 0);
		Comm.release_Nexus();
		
		lctrl_sp.put(nx_put_buf);
		lcomm_sp.put(nx_put_buf);
	    }
	    catch (Bad_Parameter_Exception e)
	    {
		throw new Internal_Error(
		    "cif.comm.I_Connection_UDP.open_Connection(): " +
		    "caught an unexpected Bad_Parameter_Exception");
	    }

	    rctrl_sp.send(nx_put_buf);
	    rctrl_sp.deactivate();

	    try
	    {
		nx_get_buf = lctrl_ep.receive(timeout);
	    }
	    catch(Comm_Would_Block_Exception e)
	    {
		throw new Internal_Error(
		    "cif.comm.I_Connection_UDP.open_Connection(): " +
		    "caught unexpected Comm_Would_Block_Exception");
	    }
	    finally
	    {
		lctrl_ep.deactivate();
	    }
	    
	    try
	    {
		rcomm_sp = new I_Startpoint();
		rcomm_sp.activate(nx_get_buf);
	    }
	    catch (Bad_Parameter_Exception e)
	    {
		throw new Comm_Failure_Exception(
		    "cif.comm.I_Connection_UDP.open_Connection():" +
		    "received a bad startpoint");
	    }
	
	}
	catch (Object_Active_Exception e)
	{
	    throw new Internal_Error(
		"cif.comm.I_Connection_UDP.open_Connection(): " +
		"caught unexpected Object_Active_Exception");
	}
	catch (Object_Inactive_Exception e)
	{
	    throw new Internal_Error(
		"cif.comm.I_Connection_UDP.open_Connection(): " +
		"caught unexpected Object_Inactive_Exception");
	}

	i_ep = lcomm_ep;
	i_sp = rcomm_sp;
	i_flow = I_FLOW_BIDIRECTIONAL;
    }
    
	    
    
    protected
    void
    open_Connection(
	GetBuffer			nx_get_buf,
	I_Endpoint_Properties		ep_properties,
	Timeout				timeout)
    throws
	Timeout_Expired_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	I_Startpoint				rctrl_sp;
	I_Startpoint				lcomm_sp;
	I_Startpoint				rcomm_sp;
	I_Endpoint				lcomm_ep;

	try
	{
	    PutBuffer				nx_put_buf;

	    try
	    {
		lcomm_ep = new I_Endpoint();
		lcomm_ep.activate(ep_properties);
	
		lcomm_sp = new I_Startpoint();
		lcomm_sp.activate(lcomm_ep);
	    }
	    catch (Bad_Parameter_Exception e)
	    {
		throw new Internal_Error(
		    "cif.comm.I_Connection_UDP.open_Connection(): " +
		    "caught an unexpected Bad_Parameter_Exception");
	    }

	    try
	    {
		rctrl_sp = new I_Startpoint();
		rctrl_sp.activate(nx_get_buf);
		
		rcomm_sp = new I_Startpoint();
		rcomm_sp.activate(nx_get_buf);
	    }
	    catch (Bad_Parameter_Exception e)
	    {
		throw new Comm_Failure_Exception(
		    "cif.comm.I_Connection_UDP.open_Connection(): " +
		    "received a bad startpoint");
	    }
	
	    try
	    {
		Nexus				nx;

		nx = Comm.acquire_Nexus();
		nx_put_buf = nx.buffer_init(lcomm_sp.sizeof(), 0);
		Comm.release_Nexus();
		
		lcomm_sp.put(nx_put_buf);
	    }
	    catch (Bad_Parameter_Exception e)
	    {
		throw new Internal_Error(
		    "cif.comm.I_Connection_UDP.open_Connection(): " +
		    "caught an unexpected buffer overrun exception");
	    }

	    rctrl_sp.send(nx_put_buf);
	    rctrl_sp.deactivate();
	}
	catch (Object_Active_Exception e)
	{
	    throw new Internal_Error(
		"cif.comm.I_Connection_UDP.open_Connection(): " +
		"caught unexpected Object_Active_Exception");
	}
	catch (Object_Inactive_Exception e)
	{
	    throw new Internal_Error(
		"cif.comm.I_Connection_UDP.open_Connection(): " +
		"caught unexpected Object_Inactive_Exception");
	}

	i_ep = lcomm_ep;
	i_sp = rcomm_sp;
	i_flow = I_FLOW_BIDIRECTIONAL;
    }
}


