/******************************************************************************
I_Connection_UUM.java

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/java/libraries/cif/comm/I_Connection_UUM.java,v $
  $Date: 1999/02/25 21:39:15 $
  $Revision: 1.2 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

package cif.comm;

import nexus.*;
import cif.toolkit.*;

/**
 * This class completes the Connection implementation for the UUM protocol.
 */
class I_Connection_UUM
extends I_Connection
{
    protected
    void
    open_Connection(
	URL					url,
	I_Endpoint_Properties			ep_properties,
	Timeout					timeout)
    throws
	Bad_Parameter_Exception,
	Timeout_Expired_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {

        /*
	 * Verify URL information
         */
        if (url.get_Hostname() == null)
        {
	   throw new Bad_Parameter_Exception(
	      1, "hostname must be specified in a connection URL");
        }

        if (url.get_Port() < 1)
        {
	  throw new Bad_Parameter_Exception(
		  1, "port number may not be zero in a connection URL");
        }


	if (url.get_Protocol().compareTo(Comm.UUM_URL_SCHEME) != 0)
	{
	    throw new Bad_Parameter_Exception(
		1, "expected URL Scheme to be " + Comm.UUM_URL_SCHEME);
	}

        UdpInfo             udp_info = new UdpInfo();
        String              parameters = url.get_Parameters(); 

	udp_info.hostname = url.get_Hostname();
	udp_info.port = url.get_Port();

	// get options
	if(parameters != null && !parameters.equals("/"))
	{
	   String       opt = ((parameters.substring(0)).trim()).concat("+");
           String       curOpt;

	  try
	  {
	   while(opt.length() > 0)
	   {
              int ndx = opt.indexOf('+') + 1;
              curOpt = opt.substring(0, ndx);
              String param = curOpt.substring(0, curOpt.indexOf('='));
	      String val = curOpt.substring(curOpt.indexOf('=')+1, curOpt.indexOf('+'));

              if(param.equals("ttl"))
	      {
                 udp_info.ttl = (byte)(new Integer(val).intValue()); 
	      }
	      else if(param.equals("loopback"))
	      {
                 if(val.equals("false"))
		 {
                     udp_info.loopback = false;
		 }
		 else
		 {
                     udp_info.loopback = true;
		 }
	      }
	      else if(param.equals("multiple_senders"))
	      {
                 if(val.equals("false"))
		 {
                     udp_info.loopback = false;
		 }
		 else
		 {
                     udp_info.loopback = true;
		 }
	      }
	      else
	      {
	         throw new Bad_Parameter_Exception(
		    1, "expected URL Scheme to be " + Comm.UUM_URL_SCHEME);
	      }

              opt = opt.substring(ndx);
	   }
          }
	  catch(Exception _le)
	  {
	    throw new Bad_Parameter_Exception(
		1, "expected URL Scheme to be " + Comm.UUM_URL_SCHEME);
	  }
	}

        try
	{
	  ep_properties.set_Protocol(udp_info);

          i_ep = new I_Endpoint(); 
	  i_ep.activate(ep_properties);

          i_sp = new I_Startpoint();
          i_sp.activate(i_ep);
        }
	catch (Bad_Parameter_Exception e)
	{
	  throw new Internal_Error(
				   "cif.comm.I_Connection_UUM.open_Connection(): " +
	  		            "caught an unexpected Bad_Parameter_Exception");
	}
        catch (Object_Active_Exception e)
	{
	    throw new Internal_Error(
			    "cif.comm.I_Connection_UUM.open_Connection(): " +
			    "caught unexpected Object_Active_Exception");
        }

	i_flow = I_FLOW_BIDIRECTIONAL;
    }
    
	    
    
    protected
    void
    open_Connection(
	GetBuffer			nx_get_buf,
	I_Endpoint_Properties		ep_properties,
	Timeout				timeout)
    throws
	Timeout_Expired_Exception,
	Comm_Inactive_Exception,
	Comm_Failure_Exception,
	InterruptedException
    {
	  throw new Internal_Error(
				   "cif.comm.I_Connection_UUM.open_Connection(): " +
	  		            "caught an unexpected Bad_Parameter_Exception");
    }
}


