package cif.shared_state;

import java.util.*;
import java.lang.*;
/**
 *   An interface that allows the user to register for event notification
 *  on a given shared State.
 *
 *  <PRE> 
 *  An object implementing the Callback_Interface interface is passed to 
 *  State.add_Callback_Interface().  The object is now registered
 *  to recieve notification of changes to that State.
 *  </PRE>
*/

//***************************************************************************
//*  CIF.Shared_State.Callback_Interface
//*  -----------------------------------
//***************************************************************************
public interface Callback_Interface extends EventListener
{
    /**
       Notification that a state is changed is given to the user via
       the state_Changed() method.  The state_Changed() method is called
       on all Callback_Interfaces registered with a state whose value
       has been altered.

       @param shared_state A reference to the State whose value change caused 
	      the event.
       
       @param shared_state_changed_type Idenfies the change event as 
              FINAL (reliable messaging:tcp) or INTERMEDIATE (unreliable 
	      multicast).

       @param guid A unique indentifier of the Set that made the change. 
	      This value enables the user to recognize what node in the
	      message space made the change.
     */
    public void state_Changed(State shared_state,
			      int shared_state_changed_type,
			      Set_GUID guid);
};
