package cif.shared_state;

/**
   An interface used to define the methods needed for a messenger object
   that can comunicate with the shared_state message space.
    
<blockquote>  
Creating a new messaging mechanism for the shared_state library is relatively
easy.  First new messenger must be created that implements this interface. 
Then the Set class must be modified in the following ways:

Add a string identifier for the class
ex:CIF_MESSENGER_TCP.
Modify the constructor to create the new Messenger when the above 
string is passed to it.

</blockquote> 


*@author John Bresnahan  
*/  
//**************************************************************************
//*  Messenger_Interface
//*  -------------------
//**************************************************************************
public interface Messenger_Interface
{
    /**
       Returns the url of the message space that the Messenger is
       connected to.
     */
    public String  get_Message_Space_URL();

    /**
       Sends a passed serial object into message space.  This method gets
       a byte array from the passed in Serial object and distributes
       it into message space.  The state_changed_type determines
       what protocol will be use.
    */
    public void    send(Serial serial, int state_changed_type);
    
    /**
       Called when a new State has been added to message space.
       The method handles all communication initialization needed,
       and notifies all other nodes of the change.
    */
    public void    add_Shared_State(State state);
    
    /**
       Called when the local program whats to leave message space.  Calling 
       this method notifies all other nodes attached to the message space
       that the local node has left.
    */
    public void    close();
    
    /**
       Creates a new serial object to be used by Sets using this 
       Messenger_Interface interface for communication.  This method creates
       a new Serial object with the necessary headers packed at the 
       begining of the buffer.
     */
    public Serial  new_Shared_State_Serial();
};

