package cif.shared_state;

import java.util.*;
import java.lang.*;
/**
   An interface that allows the user to register for event notification
   on a given Set.

<blockquote>  
An object implementing the Set_Event_Listener can register with a Set
to receive notifiction when any of the following Event occur:

A Set has joined message space.
A Set has left message space.
A State has been added to a Set in message space.
A State has changed its value in message space.

</blockquote> 
*/

/*****************************************************************************
*  CIF.Shared_State.Set_Listener_Interface
*  ---------------------------------------
*****************************************************************************/
public interface Set_Event_Listener extends EventListener
{
    /**
     *  A state in message space has made a change. 
     *  @param state The state that was changed.
     *  @param guid the ID of the Set that made the change.
     */
    public void state_Change_Occurred(String state, Set_GUID guid);
    
    /**
     *  Notification that a set has joined message space.
     *  @param guid ID of the set that joined message space.
     */
    public void set_Joined(Set_GUID guid);
    /**
     * Notification that a set has left message space.
     * @param guid ID of the set that left message space.
     */
    public void set_Exited(Set_GUID guid);
    /**
     *  Notication that a state has been added to a Set 
     *  in message space.  
     *  @param state_name The ID of the State that was added.
     *  @param source_set The ID of the Set that added the State.
     */
    public void state_Added(String state_name, 
			    Set_GUID source_set);    
};

