package cif.shared_state;

import java.lang.*;

/**
   This class uniquely identifies a set in shared space.
   The guid is built by a given byte array passed by the 
   Set associated with it.  The Set is responsible for
   providing a unique byte array for each instantiation of Set_GUID.

<blockquote>
This class allows Sets in the message to space to identify the sender
of given messages.
</blockquote>
*/
public class Set_GUID
{
  private byte  byte_Value[];
  private int   value_Length;

    /**
       Creates a GUID from the given unique byte array.
     */
    public Set_GUID(byte value[], 
		    int length)
    {
	byte_Value = new byte[length];
	value_Length = length;
	System.arraycopy(value, 0, byte_Value, 0, length);
    }

    /**
       Tests if one Set_GUIDs byte array is equal to this Set_GUIDs
       byte array.
     */
    public boolean equals(Object in_arg)
    {
	byte gb[];
	boolean eq = true;
	int ctr;
	Set_GUID guid;
	
	try
        {
	    guid = (Set_GUID) in_arg;
	}
	catch(Exception e)
        {
	    return false;
	}

	gb = guid.get_Bytes();
	
	if(gb.length != byte_Value.length) return false;
	//      return gb.equals(byte_Value);
	
	for(ctr = 0; ctr < gb.length; ctr++)
        {
	    if(byte_Value[ctr] != gb[ctr]) eq = false;
	}
	return eq;
    }

    /**
       Returns the length of the byte array.
     */
    public int get_Length()
    {
	return value_Length;  
    }
    
    /**
       returns the byte array passed into the constructor that 
       uniquly identifies this set from all others in message space.
     */
    public byte [] get_Bytes()
    {
	return byte_Value; 
    }
}



