package cif.shared_state.messenger;

import cif.shared_state.*;
import java.lang.*;
import cif.comm.*;
import cif.toolkit.*;
import java.util.Vector;
import java.util.Hashtable;
import nexus.DataConversion;


/**
   A total order message space manager.

<blockquote>
This messenger uses TCP for all FINAL messages, and UDP multicast for 
all INTERMEDIATE messages (UDP is not currently implemented).  The first
messenger in message space is designated the master.  The master listens 
for connects on a url which is published by the object via a call to 
get_Message_Space_URL().  The master keeps a list of all slaves that connect
and ensures total order between them.  
</blockquote>
 */
class L_Connection implements Event_Listener
{
    protected  Connection                internal_Conn = null;
    protected  Connection                tcp_Conn = null;
    protected  Connection                uum_Conn = null;
    protected  Messenger_TCP             messenger;
    protected  boolean                   initialized = false;
    protected  int                       connection_Count = 0;
    protected  Listener                  tcp_Listener = null;  
    protected  Listener                  uum_Listener = null;

    public static final int              INTERNAL_WIRE = 0;
    public static final int              TCP_WIRE = 1;
    public static final int              UUM_WIRE = 2;

    /*
     *
     */
    public 
    L_Connection(
        Messenger_TCP                    messenger, 
        Listener                         listener)
    {
        Listener_Properties              propertiesListener;
        String                           tcp_url;
	String                           uum_url;
        char                             tcp_chars[];
        char                             uum_chars[];
	Connection_Properties            propertiesConn;
	Serial                           serial = new Serial();
	int                              len[] = new int[1];

        this.messenger = messenger; 
        
        propertiesConn = new Connection_Properties();
        propertiesConn.set_Event_Listener(this);
        propertiesListener = new Listener_Properties();
        propertiesListener.set_Event_Listener(this);

        try
        {
           internal_Conn = listener.get_Connection(propertiesConn, 0);
        
           tcp_Listener = Factory.get_Listener("x-cif-tcp://", 
                                               propertiesListener);
	   uum_Listener = Factory.get_Listener("x-cif-tcp://",
					       propertiesListener);

           tcp_url = tcp_Listener.get_URL();
           tcp_chars = tcp_url.toCharArray();
	   uum_url = uum_Listener.get_URL();
	   uum_chars = uum_url.toCharArray();

	   len[0] = tcp_chars.length;
	   serial.pack_Int(len, 0, 1);
	   serial.pack_Char(tcp_chars, 0, len[0]);
	   len[0] = uum_chars.length;
	   serial.pack_Int(len, 0, 1);
	   serial.pack_Char(uum_chars, 0, len[0]);

	   System.out.println("tcp url:" + tcp_chars);
	   System.out.println("uum url:" + uum_chars);


           /*
            *  pack in total connection count
            */ 
           messenger.total_Connections++;
	   len[0] = messenger.total_Connections;
	   serial.pack_Int(len, 0, 1);
 
	   byte myass[] = new byte[serial.get_Size_Buffer()];
	   byte b[]  = serial.get_Buffer();
	   System.arraycopy(b, 0, myass, 0, myass.length);
	   internal_Conn.send(myass, 
			      myass.length,
			      0);

	   internal_Conn.enable_Event_Notification();
           tcp_Listener.enable_Event_Notification();
           uum_Listener.enable_Event_Notification();

        }
        catch(Exception e)
        {
           e.printStackTrace();
           return;
        }

        this.messenger = messenger; 
    }

    /*
     *
     */
    public 
    L_Connection(
        Messenger_TCP           messenger, 
        String                  url)
      throws Exception
    {
        Connection_Properties   propertiesConn = new Connection_Properties();
        Message                 msg;
        String                  tcp_url;
	String                  uum_url;
	char                    tcp_chars[];
	char                    uum_chars[];
	Serial                  serial;
	int                     len[] = new int[1];

        this.messenger = messenger;

        try
        {
          propertiesConn.set_Event_Listener(this);


          internal_Conn = Factory.get_Connection(url, 
						 propertiesConn);

          msg = internal_Conn.receive();
	  
	  // extract information
	  Message.Info info = msg.get_Info();
	  serial = new Serial(msg.get_Data(), info.size);
	  serial.unpack_Int(len, 0, 1);
	  tcp_chars = new char[len[0]];
	  serial.unpack_Char(tcp_chars, 0, len[0]);
	  tcp_url = new String(tcp_chars);
	  
	  serial.unpack_Int(len, 0, 1);
	  uum_chars = new char[len[0]];
	  serial.unpack_Char(uum_chars, 0, len[0]);
	  uum_url = new String(uum_chars);

	  serial.unpack_Int(len, 0, 1);
	  messenger.total_Connections = len[0];
	  
          tcp_Conn = Factory.get_Connection(tcp_url, propertiesConn);
	  uum_Conn = Factory.get_Connection(uum_url, propertiesConn);

          initialized = true;
          internal_Conn.enable_Event_Notification();
          tcp_Conn.enable_Event_Notification();
	  uum_Conn.enable_Event_Notification();

        }
        catch(Exception e)
        {
           e.printStackTrace();

           throw e;
        }
    }

    public synchronized void 
    enable(
        boolean                                b)
    {
	try 
        {    
	    if(b)
	    {
		tcp_Conn.enable_Event_Notification();
		uum_Conn.enable_Event_Notification();
	    }
	    else
	    {
		tcp_Conn.disable_Event_Notification();
		uum_Conn.disable_Event_Notification();
	    }
	}
	catch (Exception e) 
        {
            System.err.println("Failed to set enable to " + b);
            e.printStackTrace();
	}
    }

    /*
     *
     */
    public synchronized void 
    handle_Event(
        Event                              event)
    {
        Connection_Properties              propertiesConn;
	Connection                         conn;
	Listener                           list;

        if (event instanceof Listener_Connection_Request_Event)
        {
	    Listener_Connection_Request_Event 
                                           arrival_event = 
                         	(Listener_Connection_Request_Event) event; 
	    list = arrival_event.get_Listener();
            if(!initialized)
            {
                try
                {
                    propertiesConn = new Connection_Properties();
                    propertiesConn.set_Event_Listener(this);

		    if(list == tcp_Listener)
		    {
			tcp_Conn = tcp_Listener.get_Connection(
                                        propertiesConn,
				        0);
			tcp_Conn.enable_Event_Notification();
		    connection_Count++;
		    }
		    else if(list == uum_Listener)
		    {
			uum_Conn = uum_Listener.get_Connection(
                                       propertiesConn,
				       0);
			uum_Conn.enable_Event_Notification();
		    connection_Count++;
		    }
                    else
                    {
                        System.err.println("bad connection");
                    }

		    if(connection_Count > 1) 
		    {
			initialized = true;
		    }
                }
                catch(Exception e)
                {
                    e.printStackTrace();
                    System.err.println("failed to get connection");
                }
            }
        }
        else if(event instanceof Connection_Message_Arrival_Event)
        {
            Message       msg = null;
            byte          buf[];
            Serial        serial;
            Connection_Message_Arrival_Event arrival_event;

            arrival_event = (Connection_Message_Arrival_Event) event;
            if(initialized)
            {
               conn = arrival_event.get_Connection();

               try
               {
                    msg =  conn.receive();
               }
               catch(Exception e)
               {
                   e.printStackTrace();            
               }

               buf = msg.get_Data();
               Message.Info info = msg.get_Info();

               serial = new Serial(buf, info.size);

               if(conn == tcp_Conn)
               {
                    messenger.receive_TCP(serial);
               }
               else if(conn == internal_Conn)
               {
                    messenger.receive_Internal(serial, this);
               }
               else if(conn == uum_Conn)
               {
		   messenger.receive_UUM(serial);
               }
            }            
        }
    }


    /*
     *  could be called locked
     */
    public synchronized void 
    send( 
        Serial                            serial, 
        long                              seq_num, 
        int                               wire)
    {
        Connection                        conn;
        int                               size;
        byte                              buf[];
	long                              value[] = new long[1];

        if(wire == INTERNAL_WIRE)
	{
            conn = internal_Conn;
        }
        else if(wire == TCP_WIRE)
	{
            conn = tcp_Conn;
        }
        else if(wire == UUM_WIRE)
	{
	    conn = uum_Conn;
        }
        else
	{
            System.err.println("trying to send on a bad connection");
            return;
        }

        buf = serial.get_Buffer();
        size = serial.get_Size_Buffer();

	value[0] = seq_num;
	DataConversion.put_long(value, 0, 1, buf, 1); 

        try
        {
/*            byte temp[] = new byte[size];

            System.arraycopy(buf, 0, temp, 0, size);
            conn.send(temp, temp.length);
 */ 
            conn.send(buf, size);
        }
        catch(Exception e)
        {
	    e.printStackTrace(); 

	    System.exit(1);
        }  
    }

    public synchronized void 
    close()
    {
        try
	{
            initialized = false;

            tcp_Conn.close();
            uum_Conn.close();
            internal_Conn.close();

            tcp_Conn = null;
            uum_Conn = null;
            internal_Conn = null;
        } 
	catch (Exception e)
	{            
        }
    }
}

/******************************************************************************
*  CIF.Shared_State.Messenger_TCP
*  ------------------------------
******************************************************************************/
public class 
Messenger_TCP 
  implements Messenger_Interface,
             Event_Listener
{
    protected Set                   shared_State_Set;
    protected String                space_Url;

    private   long                  send_Number = 0;
    private   long                  last_Number = 0;
    protected boolean               master_Flag;

    protected L_Connection          slave_Conn = null;
    protected Vector                connections = null;
    protected Hashtable             naks_List = new Hashtable();

    private static final byte       STATE_CHANGE_MSG = 0;
    private static final byte       INTERNAL_STATE_QUERY_MSG = 1;
    private static final byte       INTERNAL_JOIN_MSG = 2;
    private static final byte       INTERNAL_EXIT_MSG = 3;
    private static final byte       INTERNAL_CONFIRM_EXIT_MSG = 4;
    private static final byte       MESSAGE_DROPPED_MSG = 5;
    private static final byte       INTERNAL_STATE_ACK_MSG = 6;
    private static final byte       INTERNAL_STATE_NAK_MSG = 7;
    private static final byte       INTERNAL_STATE_DUMP_REQUST = 8;
    private static final byte       INTERNAL_DUMP_ALL_REQUEST = 9;
    
    protected  Listener             connection_Listener = null;  
    private    boolean              can_Exit;

    private    int                  times_locked = 0;
    public     int                  total_Connections = 0;

    class 
    L_Header
    {
        public byte  type;
        public long  seq_Num;

        L_Header(byte type, long seq)
        {
            this.type = type;
            this.seq_Num = seq;
        }
    }

    class 
    L_Nak_Node 
    {
	private int      naks_Needed;
	private int      naks_Received;
	private State    state;
	private Set      sharedSet;
	private byte     key_Bytes[];
	private boolean  finished = false;
	
	public L_Nak_Node(State state, 
			  int nak,
			  byte key[])
	{
	    this.state = state;
	    naks_Needed = nak;
	    naks_Received = 0;
	    this.sharedSet = Messenger_TCP.this.shared_State_Set;
	    key_Bytes = key;
	} 
	
	public synchronized boolean 
        equals(
            Object                          obj)
	{
	    L_Nak_Node                      in_arg = null;
	    
	    try
	    {
		in_arg = (L_Nak_Node) obj;
	    } 
	    catch (Exception e) {return false;}
	    
	    String obj_name = in_arg.get_Name();
	    Set_GUID obj_guid = in_arg.get_GUID();

	    if(obj_name.equals(state.get_Name()) &&
	       obj_guid.equals(sharedSet.get_GUID()))
	    {
		return true;
	    }

	    return false;
	}

	public synchronized byte [] 
        get_ID()
	{
	    return key_Bytes;
	}

	public synchronized Set_GUID 
        get_GUID()
	{
	    return sharedSet.get_GUID();
	}
	
	public synchronized String 
        get_Name()
	{
	    return state.get_Name();
	}

	public synchronized boolean 
        add_Nak()
	{
	    naks_Received++;
	    if(naks_Received < naks_Needed) 
            {
                return false;
            }
	    
	    Messenger_TCP.this.remove_Nak(state.get_Name().toCharArray(),
					  key_Bytes);
	    finished = true;
	    return true;
	}
	
	public synchronized boolean 
        is_Done()
	{
	    if(naks_Received < naks_Needed) 
            {
                return false;
	    }
	    return true;
	}
    }


    /*
     *
     */
    public 
    Messenger_TCP(
        Set                                shared_state_set)
    {
        Listener_Properties properties_listener;

        Comm.activate();
        can_Exit = true;

        try {
	    properties_listener = new Listener_Properties();
	    properties_listener.set_Event_Listener(this);
	    
	    this.shared_State_Set = shared_state_set;
	    
	    connections = new Vector(5, 5);

	    connection_Listener = Factory.get_Listener("x-cif-tcp://",
						       properties_listener);

	    connection_Listener.enable_Event_Notification();
	    space_Url = connection_Listener.get_URL();
	    master_Flag = true;	    
        }
        catch(Exception e)
        {
            System.out.println("Exception ");
            e.printStackTrace();
        }
    }
    
    /*
     *
     */
    public 
    Messenger_TCP(
        Set                          shared_state_set, 
        String                       url) 
      throws Exception
    {
        L_Header header;
        int len[] = new int[1];
        byte guid_bytes[];
        Serial serial;

        can_Exit = false;
        Comm.activate();
        
        this.shared_State_Set = shared_state_set;
        space_Url = new String(url);

        slave_Conn = new L_Connection(this, url);

        master_Flag = false;

        // send set joined message
        header = new L_Header(INTERNAL_JOIN_MSG, send_Number);
        guid_bytes = shared_State_Set.get_GUID().get_Bytes();
        len[0] = guid_bytes.length;
        
        serial = create_Serial_Header(header);
        serial.pack_Int(len, 0, 1);
        serial.pack_Byte(guid_bytes, 0, guid_bytes.length);
	len[0] = total_Connections;
	serial.pack_Int(len, 0, 1);

        distribute_Message(serial, L_Connection.INTERNAL_WIRE, null);
    }

    /*
     *
     */
    public void 
    enable_Connections(
        boolean                                b)
    {
	int                                    ctr;
	L_Connection                           l_conn;

	if(b) 
        {
            times_locked--;
        }
	if(times_locked < 0) 
        {
            times_locked = 0;
        }

	if(times_locked == 0)
	{
	    if(master_Flag)
	    {
		for(ctr = 0; ctr < connections.size(); ctr++)
		{
		    l_conn = (L_Connection) connections.elementAt(ctr);
		    l_conn.enable(b);
		}
	    }
	    else
	    {
		slave_Conn.enable(b);
	    }
	}
	
	if(!b) 
        {
            times_locked++;
        }
    }

    /*
     *
     */
    private Serial 
    create_Serial_Header(
        L_Header                              header)
    {
	Serial                                serial;

        serial = new Serial();
	create_User_Serial_Header(serial, header);

        return serial;
    }

    /*
     *
     */
    private void 
    create_User_Serial_Header(
        Serial                               serial, 
        L_Header                             header)
    {
        byte type[] = new byte[1];
        long seq[] = new long[1];

        type[0] = header.type;
        seq[0] = header.seq_Num;
        serial.pack_Long(seq, 0, 1);
        serial.pack_Byte(type, 0, 1);
    }

    /*
     *
     */
    private L_Header 
    rip_Serial_Header(
        Serial                              serial)
    {
        long seq[] = new long[1];
        byte type[] = new byte[1];

        try
        {
            serial.unpack_Long(seq, 0, 1);
	    serial.unpack_Byte(type, 0, 1);
        }
        catch (Exception e)
        {
           e.printStackTrace();            
        }
        L_Header header = new L_Header(type[0], seq[0]);

        return header;
    }

    /*
     *
     */
    public synchronized String 
    get_Message_Space_URL()
    {
        return space_Url;
    }
    
    /*
     *
     */
    public synchronized void 
    add_Shared_State(
        State                                  state)
    {
        Serial serial;
        byte guid_bytes[];
        int  len[] = new int[1];
        char name[];
	int ctr;
	L_Connection l_conn;
	L_Nak_Node nak_Node;
	byte key_bytes[];
	
        
        serial = create_Serial_Header(new L_Header(INTERNAL_STATE_QUERY_MSG,
                                                   send_Number));

        // pack in the GUID
        guid_bytes = shared_State_Set.get_GUID().get_Bytes();
        len[0] = guid_bytes.length;
        serial.pack_Int(len, 0, 1);
        serial.pack_Byte(guid_bytes, 0, guid_bytes.length);

        // pack in the state name
        name = state.get_Name().toCharArray();
        len[0] = name.length;
        serial.pack_Int(len, 0, 1);
        serial.pack_Char(name, 0, name.length);
       
	if(master_Flag)
	{
	    if(total_Connections == 0)
	    {
		Serial s;
		L_Header h;

		serial = shared_State_Set.new_Shared_State_Serial(state);
		state.pack_State(serial);
		
		s = new Serial(serial.get_Buffer(), serial.get_Size_Buffer());
		h = rip_Serial_Header(s);
		shared_State_Set.receive(s, State.INITIALIZE, h.seq_Num);
		return;
	    }
	}
	byte name_bytes[] = state.get_Name().getBytes();

	enable_Connections(false);
	key_bytes = new byte[name.length + guid_bytes.length];
	System.arraycopy(name_bytes, 0, key_bytes, 0, name.length);
	System.arraycopy(guid_bytes, 0, key_bytes, 
			 name.length, guid_bytes.length);

	nak_Node = new L_Nak_Node(state, total_Connections, key_bytes);
	
	naks_List.put(new String(key_bytes), nak_Node);
	distribute_Message(serial, L_Connection.INTERNAL_WIRE, null);
    }
    
    /*
     *
     */
    public synchronized void
    send(
        Serial serial, 
        int state_changed_type)
    {
        int wire;

        if(state_changed_type == State.FINAL)
        {
            wire = L_Connection.TCP_WIRE;
        }
        else
        {
            wire = L_Connection.UUM_WIRE;
        }

        if(master_Flag)
        {
	    L_Header h;

	    send_Number++;
            Serial s = new Serial(serial.get_Buffer(),
                                  serial.get_Size_Buffer());
            h = rip_Serial_Header(s);
            shared_State_Set.receive(s, state_changed_type, h.seq_Num);
        }
        distribute_Message(serial, wire, null);
    }
    
    /*
     *
     */
    public Serial 
    new_Shared_State_Serial()
    {
	Serial serial;

        L_Header header = new L_Header(STATE_CHANGE_MSG,
                                       send_Number);
        serial = create_Serial_Header(header);

	return serial;
    }

    /*
     *
     */   
    public void 
    receive_TCP(Serial serial)
    {
        L_Header header;

        if(master_Flag)
        {
            distribute_Message(serial, L_Connection.TCP_WIRE, null);
        }
        
        header = rip_Serial_Header(serial);

	last_Number = header.seq_Num;
        shared_State_Set.receive(serial, State.FINAL, header.seq_Num);
    }

    /*
     *
     */   
    public void 
    receive_UUM(
        Serial serial)
    {
//        receive_TCP(serial);
    }

    /*
     *
     */   
    public void 
    receive_Internal(Serial serial, L_Connection l_conn)
    {
        byte guid_bytes[];
        char name[];
        State state;
        L_Header header;
        int name_len[] = new int[1];
	int guid_len[] = new int[1];
        int ctr;
        Set_GUID guid = null;
        Serial s;
	byte key_bytes[];
	L_Nak_Node nak_Node;


        if(master_Flag)
        {
            distribute_Message(serial, L_Connection.INTERNAL_WIRE, l_conn);
        }

        header = rip_Serial_Header(serial);
        try{

	    serial.unpack_Int(guid_len, 0, 1);
	    guid_bytes = new byte[guid_len[0]];
	    serial.unpack_Byte(guid_bytes, 0, guid_len[0]);
	    guid = new Set_GUID(guid_bytes, guid_bytes.length);
	    
	switch(header.type)
        {
        case INTERNAL_STATE_QUERY_MSG:
                
            serial.unpack_Int(name_len, 0, 1);
            name = new char[name_len[0]];
            serial.unpack_Char(name, 0, name.length);

	    state = shared_State_Set.get_Shared_State(new String(name));
	    // send around and ACK
	    if(state != null)
	    {
		L_Header ack_header = new L_Header(INTERNAL_STATE_ACK_MSG,
						    send_Number);
                
		s = create_Serial_Header(ack_header);
		s.pack_Int(guid_len, 0, 1);
		s.pack_Byte(guid_bytes, 0, guid_bytes.length);
		s.pack_Int(name_len, 0, 1);
		s.pack_Char(name, 0, name.length);
		shared_State_Set.pack_User_Set(s, state);
		state.pack_State(s);

		distribute_Message(s, L_Connection.INTERNAL_WIRE, null);
	    }
	    // send around a NAK
	    else
	    {
		L_Header nak_header = new L_Header(INTERNAL_STATE_NAK_MSG,
						    send_Number);
                
		s = create_Serial_Header(nak_header);
		s.pack_Int(guid_len, 0, 1);
		s.pack_Byte(guid_bytes, 0, guid_bytes.length);
		s.pack_Int(name_len, 0, 1);
		s.pack_Char(name, 0, name.length);

		distribute_Message(s, L_Connection.INTERNAL_WIRE, null);
	    }

            break;

	case INTERNAL_STATE_ACK_MSG:
	    char ack_name[];
	    int  ack_name_len[] = new int[1];

	    serial.unpack_Int(ack_name_len, 0, 1);
	    ack_name = new char[ack_name_len[0]];
	    serial.unpack_Char(ack_name, 0, ack_name_len[0]);

	    byte ack_name_bytes[] = new String(ack_name).getBytes();

	    key_bytes = new byte[ack_name_bytes.length+guid_bytes.length];
	    System.arraycopy(ack_name_bytes, 0, key_bytes, 
			     0, ack_name_bytes.length);
	    System.arraycopy(guid_bytes, 0, key_bytes, 
			     ack_name_bytes.length, guid_bytes.length);

	    nak_Node = (L_Nak_Node) naks_List.get(new String(key_bytes));
	    
	    if(nak_Node != null)
	    {
		naks_List.remove(ack_name);
		shared_State_Set.receive(serial, 
					 State.INITIALIZE, 
					 header.seq_Num);
		enable_Connections(true);
	    }

	    break;

	case INTERNAL_STATE_NAK_MSG:
	    char nak_name[];
	    int  nak_name_len[] = new int[1];

	    serial.unpack_Int(nak_name_len, 0, 1);
	    nak_name = new char[nak_name_len[0]];
	    serial.unpack_Char(nak_name, 0, nak_name_len[0]);


	    byte nak_name_bytes[] = new String(nak_name).getBytes();

	    key_bytes = new byte[nak_name_bytes.length+guid_bytes.length];
	    System.arraycopy(nak_name_bytes, 0, key_bytes, 
			     0, nak_name_bytes.length);
	    System.arraycopy(guid_bytes, 0, key_bytes, 
			     nak_name_bytes.length, guid_bytes.length); 
	    nak_Node = (L_Nak_Node) naks_List.get(new String(key_bytes));

	    if(nak_Node != null)
	    {
		if(nak_Node.add_Nak())
		{
                    naks_List.remove(nak_name);

                    enable_Connections(true);
		}
	    }

	    break;
        
        case  INTERNAL_JOIN_MSG:

	    int c_count[] = new int[1];
	    
	    serial.unpack_Int(c_count, 0, 1);
	   
	    if(master_Flag)
	    {
//		total_Connections++;
	    }
            else
            {
	        total_Connections = c_count[0];
            }
	    
	    shared_State_Set.set_Joined(guid);

            break;

        case  INTERNAL_EXIT_MSG:        

            L_Header kill_header = new L_Header(INTERNAL_CONFIRM_EXIT_MSG,
                                                send_Number);
                
            s = create_Serial_Header(kill_header);
	    guid_len[0] = shared_State_Set.get_GUID().get_Length();
	    guid_bytes = shared_State_Set.get_GUID().get_Bytes();
            s.pack_Int(guid_len, 0, 1);
            s.pack_Byte(guid_bytes, 0, guid_bytes.length);

            l_conn.send(s, 3200, L_Connection.INTERNAL_WIRE);
                
	    shared_State_Set.set_Exited(guid);

            if(master_Flag)
            {
		total_Connections--;
                connections.removeElement(l_conn);
            }

            break;
           
        case  INTERNAL_CONFIRM_EXIT_MSG:

            can_Exit = true;
            synchronized(this)
            {
                this.notify();
            }
            break;
           
        }

        }
        catch(Exception e) 
        {
            e.printStackTrace();
        }
    }

    protected void 
    remove_Nak(char name[], byte key_bytes[])
    {
	State    state;
	Serial   serial;
	Serial   s;
	L_Header header;


	state = shared_State_Set.get_Shared_State(new 
						  String(name));
	serial = shared_State_Set.new_Shared_State_Serial(state);
	state.pack_State(serial);
	
	s = new Serial(serial.get_Buffer(), 
		       serial.get_Size_Buffer());
	header = rip_Serial_Header(s);
	shared_State_Set.receive(s, 
				 State.NAK_INITIALIZE,
				 header.seq_Num);
	
	naks_List.remove(new String(key_bytes));
	
	enable_Connections(true);	
    } 

 
    /*
     *
     */
    private synchronized void 
    distribute_Message(
        Serial                                    serial,
        int                                       wire,
        L_Connection                              skip_conn)
    {
        int ctr;
        L_Connection l_conn;

        try {
        if(!master_Flag)
        {
            slave_Conn.send(serial, send_Number, wire);
            return;
        }

        for(ctr = 0; ctr < connections.size(); ctr++)
        {
            l_conn = (L_Connection) connections.elementAt(ctr);
            if(l_conn != skip_conn)
            {
                l_conn.send(serial, send_Number, wire);
            }
        }
        }
        catch (Exception e)
        {
           e.printStackTrace();            
        }
    }

    /*
     *
     */
    public synchronized void 
    handle_Event(
        Event                                       event)
    {
        L_Connection l_conn;

        try
	{
	    if (event instanceof Listener_Connection_Request_Event)
	    {
		l_conn = new L_Connection(this, connection_Listener);
		
		connections.addElement(l_conn);
	    }
	    else 
	    {
		System.out.println("Unknown event occured");
	    }
        }
        catch (Exception e)
        {
           e.printStackTrace();            
        }
    }

    public synchronized void 
    close()
    {
        byte                             guid_bytes[];
        int                              len[] = new int[1];
        Serial                           serial;
        L_Header                         header;
        
        guid_bytes = shared_State_Set.get_GUID().get_Bytes();
        len[0] = guid_bytes.length;
        header = new L_Header(INTERNAL_EXIT_MSG, send_Number);
        serial = create_Serial_Header(header);
        serial.pack_Int(len, 0, 1);
        serial.pack_Byte(guid_bytes, 0, guid_bytes.length);
        
        //distribute_Message(serial);
        slave_Conn.send(serial, 3, L_Connection.INTERNAL_WIRE);
   
        while(!can_Exit)
        {
            try
            {
                this.wait();
            }
            catch (Exception e)
            {
                System.err.println("Exception while waiting");
                e.printStackTrace();
            }
        }
        
        if(!master_Flag) slave_Conn.close();
        Comm.deactivate();
    }
}


