/******************************************************************************
URL.java

CVS Information:

  $Source: /home/dsl/Master/cif/CIF-Current/java/libraries/cif/toolkit/URL.java,v $
  $Date: 1999/02/26 04:53:57 $
  $Revision: 1.6 $
  $State: Exp $
  $Author: bresnaha $
******************************************************************************/

package cif.toolkit;


public
class URL
{
    /**
     *
     */
    public
    URL(
	String				url)
    throws
	Bad_Parameter_Exception
    {
	/*
	 * Initialize member fields
	 */
	protocol = null;
        address = null;	
	parameters = null;
	
	/*
	 * Extract protocol field out of URL
	 */
	int				pos;

	url = new String(url);
	url = url.trim();
	pos = url.indexOf(':');
	if (pos == -1)
	{
	    throw new
		Bad_Parameter_Exception(1, "bad protocol field");
	}
        

	try
	{
	    protocol = url.substring(0, pos);
	    url = url.substring(pos + 1);
	}
	catch (StringIndexOutOfBoundsException e)
	{
	    throw new
		Internal_Error("URL.URL() caught an unexpected " +
			       "StringIndexOutOfBoundsException");
	}
        
	
	/*
	 * If nothing remains, then URL precisely matches the form of
	 * "x-cif-*:"
	 */
	if (url.length() == 0)
	{
	    return;
	}
	

	/*
	 * Looking for "//" before hostname; if it's not found, then the URL
	 * doesn't match the final acceptable form of
	 * "x-cif-*://[host][:port][/...]"
	 */
	if (url.regionMatches(0, "//", 0, 2) == false)
	{
	    throw new
		Bad_Parameter_Exception(1, "incorrect format");
	}

	try
	{
	    url = url.substring(2, url.length());
	}
	catch (StringIndexOutOfBoundsException e)
	{
	    throw new
		Internal_Error("URL.URL() caught an unexpected " +
			       "StringIndexOutOfBoundsException");
	}
	
	
	/*
	 * Extract host and port fields, and protocol parameters
	 */

	 
	int				pos_slash;

	pos_slash = url.indexOf('/');
	if (pos_slash == -1)
	{
	    pos_slash = url.length();
	    parameters = null;
	}
	else
	{
            parameters = url.substring(pos_slash+1);
	}
        address = url.substring(0, pos_slash);

    }


    /**
     *
     */
    public
    String
    get_Protocol()
    {
	return protocol;
    }


    /**
     *
     */
    public
    String
    get_Hostname()
    {
	int colon_pos = address.indexOf(':');

	if(colon_pos < 1) 
	{
	  return null;
        }
	return address.substring(0, colon_pos);
    }

    /**
     *
     */
    public 
    String
    get_Address()
    {
       return address;
    }
    
    /**
     *
     */
    public
    int
    get_Port()
    {
	int colon_pos = address.indexOf(':');
        String s;

	if(colon_pos < 1) return 0;

	s = address.substring(colon_pos+1);
        try
	{
           port = new Integer(s).intValue();
	   if(port < 0 || port > 65535)
	   {
	      return -1;
           }
	   return port;
	}
	catch(Exception e)
	{
	   return -1;
        }

    }

    
    /**
     *
     */
    public
    String
    get_Parameters()
    {
	return parameters;
    }
    

    /**
     *
     */
    public
    String
    get_String()
    {
	String					url;

	url = protocol + "://";
	if (host != null)
	{
	    url = url + host;
	}

	if (port > 0)
	{
	    url = url + ":" + port;
	}

	if (parameters != null)
	{
	    url = url + "/" + parameters;
	}

	return url;
    }


    /**************************************************************************
    **************************************************************************/

    
    /*
     * Internal data members
     */
    private String			protocol;
    private String			host;
    private int				port;
    private String			parameters;
    private String                      address;
    
    /**************************************************************************
    **************************************************************************/

    public static
    void
    main(String [] args)
    {
	URL				url;
	
	try
	{
	    url = new URL(args[0]);
	}
	catch (Bad_Parameter_Exception e)
	{
	    System.err.println(
		"Bad Parameter Exception (" + e.get_Parameter_Ordinal()
		+ "): " + e.getMessage() + "\n");
	    e.printStackTrace();
	    return;
	}

	System.out.println("String:    " + url.get_String());
	System.out.println("Protocol:  " + url.get_Protocol());
	System.out.println("Hostname:  " + url.get_Hostname());
	System.out.println("Port:      " + url.get_Port());
	System.out.println("Parameters: " + url.get_Parameters());
    }
    
}
