
package nexus;

import java.util.*;
import java.net.*;
import java.io.*;

class UdpMessage
{
    public int         fromPort;
    public InetAddress fromAddr;
    public long        endpointID;

    private int        totalSize;
    private int        expectedSeqNum;
    private int        offset;
    private byte[]     message;
    private Nexus      nexus;


    public
    UdpMessage(
	int					inSize,
	int					seq_num,
	UdpPacket				packet,
	Nexus					inNexus)
    {
	totalSize = inSize;
	fromAddr = packet.getAddress();
	fromPort = packet.getPort();
	endpointID = packet.getEndpointID();
	expectedSeqNum = seq_num;
	message = new byte[inSize];
	offset = 0;
	nexus = inNexus;
    }

    public static
    byte []
    packHeader(
	boolean					frag0,
	int					seq_num,
	int					size)
    {
	byte b[];
	byte tempB;

	b = new byte[size + UdpProto.UDP_HDR_SIZE];

	b[0] = (byte) UdpProto.PROTOCOL_VERSION;
	if(frag0) tempB = 1;
	else      tempB = 0;
	b[1] = (byte) ((tempB << 7) | ((seq_num >> 16) & 0x7f));
	b[2] = (byte) ((seq_num >> 8) & 0xff);
	b[3] = (byte) ((seq_num) & 0xff);

	return b;
    }


    /*  return true if object is to be destroyed.
	false if it is still working. */
    public boolean
    addBytes(
	byte					b[],
	int					size,
	int					seqNum)
    {
	if(seqNum != expectedSeqNum)
	{
	    return true;
	}
	try
	{
	    System.arraycopy(b, UdpProto.UDP_HDR_SIZE, message, offset,
			     size - UdpProto.UDP_HDR_SIZE);
	    offset += size;
	}
	catch(Exception e)
	{
	    return true;
	}

	if(offset >= totalSize)
	{
	    nexus.buffer_dispatch(message, endpointID);
	    return true;
	}

	expectedSeqNum++;
	return false;
    }
}

